@@ -15,12 +15,12 @@
 use std::collections::HashSet;
 use std::fmt;
 use std::io::Write as _;
-use std::str::FromStr;
 
 use clap::builder::NonEmptyStringValueParser;
 use itertools::Itertools;
-use jj_lib::backend::{CommitId, ObjectId};
+use jj_lib::backend::CommitId;
 use jj_lib::git;
+use jj_lib::object_id::ObjectId;
 use jj_lib::op_store::{RefTarget, RemoteRef};
 use jj_lib::repo::Repo;
 use jj_lib::revset::{self, RevsetExpression};
@@ -29,7 +29,7 @@ use jj_lib::view::View;
 
 use crate::cli_util::{
     parse_string_pattern, user_error, user_error_with_hint, CommandError, CommandHelper,
-    RevisionArg,
+    RemoteBranchName, RemoteBranchNamePattern, RevisionArg,
 };
 use crate::formatter::Formatter;
 use crate::ui::Ui;
@@ -39,7 +39,7 @@ use crate::ui::Ui;
 /// For information about branches, see
 /// https://github.com/martinvonz/jj/blob/main/docs/branches.md.
 #[derive(clap::Subcommand, Clone, Debug)]
-pub enum BranchSubcommand {
+pub enum BranchCommand {
     #[command(visible_alias("c"))]
     Create(BranchCreateArgs),
     #[command(visible_alias("d"))]
@@ -52,6 +52,7 @@ pub enum BranchSubcommand {
     Rename(BranchRenameArgs),
     #[command(visible_alias("s"))]
     Set(BranchSetArgs),
+    #[command(visible_alias("t"))]
     Track(BranchTrackArgs),
     Untrack(BranchUntrackArgs),
 }
@@ -202,67 +203,6 @@ pub struct BranchUntrackArgs {
     pub names: Vec<RemoteBranchNamePattern>,
 }
 
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct RemoteBranchName {
-    pub branch: String,
-    pub remote: String,
-}
-
-impl fmt::Display for RemoteBranchName {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let RemoteBranchName { branch, remote } = self;
-        write!(f, "{branch}@{remote}")
-    }
-}
-
-#[derive(Clone, Debug)]
-pub struct RemoteBranchNamePattern {
-    pub branch: StringPattern,
-    pub remote: StringPattern,
-}
-
-impl FromStr for RemoteBranchNamePattern {
-    type Err = String;
-
-    fn from_str(src: &str) -> Result<Self, Self::Err> {
-        // The kind prefix applies to both branch and remote fragments. It's
-        // weird that unanchored patterns like substring:branch@remote is split
-        // into two, but I can't think of a better syntax.
-        // TODO: should we disable substring pattern? what if we added regex?
-        let (maybe_kind, pat) = src
-            .split_once(':')
-            .map_or((None, src), |(kind, pat)| (Some(kind), pat));
-        let to_pattern = |pat: &str| {
-            if let Some(kind) = maybe_kind {
-                StringPattern::from_str_kind(pat, kind).map_err(|err| err.to_string())
-            } else {
-                Ok(StringPattern::exact(pat))
-            }
-        };
-        // TODO: maybe reuse revset parser to handle branch/remote name containing @
-        let (branch, remote) = pat
-            .rsplit_once('@')
-            .ok_or_else(|| "remote branch must be specified in branch@remote form".to_owned())?;
-        Ok(RemoteBranchNamePattern {
-            branch: to_pattern(branch)?,
-            remote: to_pattern(remote)?,
-        })
-    }
-}
-
-impl RemoteBranchNamePattern {
-    pub fn is_exact(&self) -> bool {
-        self.branch.is_exact() && self.remote.is_exact()
-    }
-}
-
-impl fmt::Display for RemoteBranchNamePattern {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let RemoteBranchNamePattern { branch, remote } = self;
-        write!(f, "{branch}@{remote}")
-    }
-}
-
 fn make_branch_term(branch_names: &[impl fmt::Display]) -> String {
     match branch_names {
         [branch_name] => format!("branch {}", branch_name),
@@ -273,17 +213,17 @@ fn make_branch_term(branch_names: &[impl fmt::Display]) -> String {
 pub fn cmd_branch(
     ui: &mut Ui,
     command: &CommandHelper,
-    subcommand: &BranchSubcommand,
+    subcommand: &BranchCommand,
 ) -> Result<(), CommandError> {
     match subcommand {
-        BranchSubcommand::Create(sub_args) => cmd_branch_create(ui, command, sub_args),
-        BranchSubcommand::Rename(sub_args) => cmd_branch_rename(ui, command, sub_args),
-        BranchSubcommand::Set(sub_args) => cmd_branch_set(ui, command, sub_args),
-        BranchSubcommand::Delete(sub_args) => cmd_branch_delete(ui, command, sub_args),
-        BranchSubcommand::Forget(sub_args) => cmd_branch_forget(ui, command, sub_args),
-        BranchSubcommand::Track(sub_args) => cmd_branch_track(ui, command, sub_args),
-        BranchSubcommand::Untrack(sub_args) => cmd_branch_untrack(ui, command, sub_args),
-        BranchSubcommand::List(sub_args) => cmd_branch_list(ui, command, sub_args),
+        BranchCommand::Create(sub_args) => cmd_branch_create(ui, command, sub_args),
+        BranchCommand::Rename(sub_args) => cmd_branch_rename(ui, command, sub_args),
+        BranchCommand::Set(sub_args) => cmd_branch_set(ui, command, sub_args),
+        BranchCommand::Delete(sub_args) => cmd_branch_delete(ui, command, sub_args),
+        BranchCommand::Forget(sub_args) => cmd_branch_forget(ui, command, sub_args),
+        BranchCommand::Track(sub_args) => cmd_branch_track(ui, command, sub_args),
+        BranchCommand::Untrack(sub_args) => cmd_branch_untrack(ui, command, sub_args),
+        BranchCommand::List(sub_args) => cmd_branch_list(ui, command, sub_args),
     }
 }
 
@@ -362,6 +302,27 @@ fn cmd_branch_rename(
             make_branch_term(&[new_branch]),
         ),
     )?;
+
+    let view = workspace_command.repo().view();
+    if view
+        .remote_branches_matching(
+            &StringPattern::exact(old_branch),
+            &StringPattern::everything(),
+        )
+        .any(|(_, remote_ref)| remote_ref.is_tracking())
+    {
+        writeln!(
+            ui.warning(),
+            "Warning: Branch {old_branch} has tracking remote branches which were not renamed."
+        )?;
+        writeln!(
+            ui.hint(),
+            "Hint: to rename the branch on the remote, you can `jj git push --branch \
+             {old_branch}` first (to delete it on the remote), and then `jj git push --branch \
+             {new_branch}`. `jj git push --all` would also be sufficient."
+        )?;
+    }
+
     Ok(())
 }
 
@@ -744,20 +705,34 @@ fn cmd_branch_list(
             if local_target.is_present() && !synced {
                 let remote_added_ids = remote_ref.target.added_ids().cloned().collect_vec();
                 let local_added_ids = local_target.added_ids().cloned().collect_vec();
-                let remote_ahead_count =
-                    revset::walk_revs(repo.as_ref(), &remote_added_ids, &local_added_ids)?.count();
-                let local_ahead_count =
-                    revset::walk_revs(repo.as_ref(), &local_added_ids, &remote_added_ids)?.count();
-                if remote_ahead_count != 0 && local_ahead_count == 0 {
-                    write!(formatter, " (ahead by {remote_ahead_count} commits)")?;
-                } else if remote_ahead_count == 0 && local_ahead_count != 0 {
-                    write!(formatter, " (behind by {local_ahead_count} commits)")?;
-                } else if remote_ahead_count != 0 && local_ahead_count != 0 {
-                    write!(
-                        formatter,
-                        " (ahead by {remote_ahead_count} commits, behind by {local_ahead_count} \
-                         commits)"
-                    )?;
+                let (remote_ahead_lower, remote_ahead_upper) =
+                    revset::walk_revs(repo.as_ref(), &remote_added_ids, &local_added_ids)?
+                        .count_estimate();
+                let (local_ahead_lower, local_ahead_upper) =
+                    revset::walk_revs(repo.as_ref(), &local_added_ids, &remote_added_ids)?
+                        .count_estimate();
+                let remote_ahead_message = match remote_ahead_upper {
+                    Some(0) => None,
+                    Some(upper) if upper == remote_ahead_lower => {
+                        Some(format!("ahead by {remote_ahead_lower} commits"))
+                    }
+                    _ => Some(format!("ahead by at least {remote_ahead_lower} commits")),
+                };
+                let local_ahead_message = match local_ahead_upper {
+                    Some(0) => None,
+                    Some(upper) if upper == local_ahead_lower => {
+                        Some(format!("behind by {local_ahead_lower} commits"))
+                    }
+                    _ => Some(format!("behind by at least {local_ahead_lower} commits")),
+                };
+                match (remote_ahead_message, local_ahead_message) {
+                    (Some(rm), Some(lm)) => {
+                        write!(formatter, " ({rm}, {lm})")?;
+                    }
+                    (Some(m), None) | (None, Some(m)) => {
+                        write!(formatter, " ({m})")?;
+                    }
+                    (None, None) => { /* do nothing */ }
                 }
             }
             print_branch_target(formatter, &remote_ref.target)?;
