@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use jj_lib::backend::ObjectId;
+use jj_lib::object_id::ObjectId;
 use tracing::instrument;
 
 use crate::cli_util::{join_message_paragraphs, CommandError, CommandHelper, RevisionArg};
@@ -41,6 +41,14 @@ pub(crate) fn cmd_checkout(
     command: &CommandHelper,
     args: &CheckoutArgs,
 ) -> Result<(), CommandError> {
+    writeln!(
+        ui.warning(),
+        "warning: `jj checkout` is deprecated; use `jj new` instead, which is equivalent"
+    )?;
+    writeln!(
+        ui.warning(),
+        "warning: `jj checkout` will be removed in a future version, and this will be a hard error"
+    )?;
     let mut workspace_command = command.workspace_helper(ui)?;
     let target = workspace_command.resolve_single_rev(&args.revision, ui)?;
     let mut tx = workspace_command.start_transaction();
