@@ -13,41 +13,43 @@
 // limitations under the License.
 
 use std::collections::HashSet;
-use std::io::{Read, Write};
+use std::io::Write;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
-use std::process::{Command, Stdio};
-use std::sync::Mutex;
-use std::time::Instant;
+use std::sync::Arc;
 use std::{fmt, fs, io};
 
 use clap::{ArgGroup, Subcommand};
 use itertools::Itertools;
-use jj_lib::backend::{ObjectId, TreeValue};
+use jj_lib::backend::TreeValue;
+use jj_lib::file_util;
 use jj_lib::git::{
     self, parse_gitmodules, GitBranchPushTargets, GitFetchError, GitFetchStats, GitPushError,
 };
-use jj_lib::git_backend::GitBackend;
+use jj_lib::object_id::ObjectId;
 use jj_lib::op_store::RefTarget;
 use jj_lib::refs::{
     classify_branch_push_action, BranchPushAction, BranchPushUpdate, TrackingRefPair,
 };
-use jj_lib::repo::Repo;
+use jj_lib::repo::{ReadonlyRepo, Repo};
 use jj_lib::repo_path::RepoPath;
 use jj_lib::revset::{self, RevsetExpression, RevsetIteratorExt as _};
 use jj_lib::settings::{ConfigResultExt as _, UserSettings};
-use jj_lib::store::Store;
 use jj_lib::str_util::StringPattern;
 use jj_lib::view::View;
 use jj_lib::workspace::Workspace;
 use maplit::hashset;
 
 use crate::cli_util::{
-    parse_string_pattern, print_failed_git_export, print_git_import_stats,
-    resolve_multiple_nonempty_revsets, short_change_hash, short_commit_hash, user_error,
-    user_error_with_hint, CommandError, CommandHelper, RevisionArg, WorkspaceCommandHelper,
+    parse_string_pattern, print_trackable_remote_branches, resolve_multiple_nonempty_revsets,
+    short_change_hash, short_commit_hash, start_repo_transaction, user_error, user_error_with_hint,
+    user_error_with_hint_opt, user_error_with_message, CommandError, CommandHelper, RevisionArg,
+    WorkspaceCommandHelper,
+};
+use crate::git_util::{
+    get_git_repo, is_colocated_git_workspace, print_failed_git_export, print_git_import_stats,
+    with_remote_git_callbacks,
 };
-use crate::progress::Progress;
 use crate::ui::Ui;
 
 /// Commands for working with the underlying Git repo
@@ -58,6 +60,7 @@ use crate::ui::Ui;
 pub enum GitCommand {
     #[command(subcommand)]
     Remote(GitRemoteCommand),
+    Init(GitInitArgs),
     Fetch(GitFetchArgs),
     Clone(GitCloneArgs),
     Push(GitPushArgs),
@@ -107,6 +110,45 @@ pub struct GitRemoteRenameArgs {
 #[derive(clap::Args, Clone, Debug)]
 pub struct GitRemoteListArgs {}
 
+/// Create a new Git backed repo.
+#[derive(clap::Args, Clone, Debug)]
+pub struct GitInitArgs {
+    /// The destination directory where the `jj` repo will be created.
+    /// If the directory does not exist, it will be created.
+    /// If no directory is diven, the current directory is used.
+    ///
+    /// By default the `git` repo is under `$destination/.jj`
+    #[arg(default_value = ".", value_hint = clap::ValueHint::DirPath)]
+    destination: String,
+
+    /// Specifies that the `jj` repo should also be a valid
+    /// `git` repo, allowing the use of both `jj` and `git` commands
+    /// in the same directory.
+    ///
+    /// This is done by placing the backing git repo into a `.git` directory
+    /// in the root of the `jj` repo along with the `.jj` directory.
+    ///
+    /// This option is only valid when creating new repos. To
+    /// reuse an existing `.git` directory in an existing git
+    /// repo, see the `--git-repo` param below.
+    ///
+    /// This option is mutually exclusive with `--git-repo`.
+    #[arg(long, conflicts_with = "git_repo")]
+    colocate: bool,
+
+    /// Specifies a path to an **existing** git repository to be
+    /// used as the backing git repo for the newly created `jj` repo.
+    ///
+    /// If the specified `--git-repo` path happens to be the same as
+    /// the `jj` repo path (both .jj and .git directories are in the
+    /// same working directory), then both `jj` and `git` commands
+    /// will work on the same repo. This is called a co-located repo.
+    ///
+    /// This option is mutually exclusive with `--colocate`.
+    #[arg(long, conflicts_with = "colocate", value_hint = clap::ValueHint::DirPath)]
+    git_repo: Option<String>,
+}
+
 /// Fetch from a Git remote
 #[derive(clap::Args, Clone, Debug)]
 pub struct GitFetchArgs {
@@ -149,25 +191,36 @@ pub struct GitCloneArgs {
 /// branch names based on the change IDs of specific commits.
 #[derive(clap::Args, Clone, Debug)]
 #[command(group(ArgGroup::new("specific").args(&["branch", "change", "revisions"]).multiple(true)))]
-#[command(group(ArgGroup::new("what").args(&["all", "deleted"]).conflicts_with("specific")))]
+#[command(group(ArgGroup::new("what").args(&["all", "deleted", "tracked"]).conflicts_with("specific")))]
 pub struct GitPushArgs {
     /// The remote to push to (only named remotes are supported)
     #[arg(long)]
     remote: Option<String>,
-    /// Push only this branch (can be repeated)
+    /// Push only this branch, or branches matching a pattern (can be repeated)
     ///
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
-    /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
+    /// https://martinvonz.github.io/jj/latest/revsets#string-patterns.
     #[arg(long, short, value_parser = parse_string_pattern)]
     branch: Vec<StringPattern>,
     /// Push all branches (including deleted branches)
     #[arg(long)]
     all: bool,
+    /// Push all tracked branches (including deleted branches)
+    ///
+    /// This usually means that the branch was already pushed to or fetched from
+    /// the relevant remote. For details, see
+    /// https://martinvonz.github.io/jj/latest/branches#remotes-and-tracked-branches
+    #[arg(long)]
+    tracked: bool,
     /// Push all deleted branches
+    ///
+    /// Only tracked branches can be successfully deleted on the remote. A
+    /// warning will be printed if any untracked branches on the remote
+    /// correspond to missing local branches.
     #[arg(long)]
     deleted: bool,
-    /// Push branches pointing to these commits
+    /// Push branches pointing to these commits (can be repeated)
     #[arg(long, short)]
     revisions: Vec<RevisionArg>,
     /// Push this commit by creating a branch based on its change ID (can be
@@ -211,13 +264,6 @@ fn make_branch_term(branch_names: &[impl fmt::Display]) -> String {
     }
 }
 
-fn get_git_repo(store: &Store) -> Result<git2::Repository, CommandError> {
-    match store.backend_impl().downcast_ref::<GitBackend>() {
-        None => Err(user_error("The repo is not backed by a git repo")),
-        Some(git_backend) => Ok(git_backend.open_git_repo()?),
-    }
-}
-
 fn map_git_error(err: git2::Error) -> CommandError {
     if err.class() == git2::ErrorClass::Ssh {
         let hint =
@@ -230,7 +276,7 @@ fn map_git_error(err: git2::Error) -> CommandError {
                  /dev/null` to the host work?"
             };
 
-        user_error_with_hint(err.to_string(), hint)
+        user_error_with_hint(err, hint)
     } else {
         user_error(err.to_string())
     }
@@ -245,7 +291,7 @@ pub fn maybe_add_gitignore(workspace_command: &WorkspaceCommandHelper) -> Result
                 .join(".gitignore"),
             "/*\n",
         )
-        .map_err(|e| user_error(format!("Failed to write .jj/.gitignore file: {e}")))
+        .map_err(|e| user_error_with_message("Failed to write .jj/.gitignore file", e))
     } else {
         Ok(())
     }
@@ -320,6 +366,131 @@ fn cmd_git_remote_list(
     Ok(())
 }
 
+pub fn git_init(
+    ui: &mut Ui,
+    command: &CommandHelper,
+    workspace_root: &Path,
+    colocate: bool,
+    git_repo: Option<&str>,
+) -> Result<(), CommandError> {
+    let cwd = command.cwd().canonicalize().unwrap();
+    let relative_wc_path = file_util::relative_path(&cwd, workspace_root);
+
+    if colocate {
+        let (workspace, repo) = Workspace::init_colocated_git(command.settings(), workspace_root)?;
+        let workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
+        maybe_add_gitignore(&workspace_command)?;
+        return Ok(());
+    }
+
+    if let Some(git_store_str) = git_repo {
+        let git_store_path = cwd.join(git_store_str);
+        let (workspace, repo) =
+            Workspace::init_external_git(command.settings(), workspace_root, &git_store_path)?;
+        // Import refs first so all the reachable commits are indexed in
+        // chronological order.
+        let colocated = is_colocated_git_workspace(&workspace, &repo);
+        let repo = init_git_refs(ui, command, repo, colocated)?;
+        let mut workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
+        maybe_add_gitignore(&workspace_command)?;
+        workspace_command.maybe_snapshot(ui)?;
+        if !workspace_command.working_copy_shared_with_git() {
+            let mut tx = workspace_command.start_transaction();
+            jj_lib::git::import_head(tx.mut_repo())?;
+            if let Some(git_head_id) = tx.mut_repo().view().git_head().as_normal().cloned() {
+                let git_head_commit = tx.mut_repo().store().get_commit(&git_head_id)?;
+                tx.check_out(&git_head_commit)?;
+            }
+            if tx.mut_repo().has_changes() {
+                tx.finish(ui, "import git head")?;
+            }
+        }
+        print_trackable_remote_branches(ui, workspace_command.repo().view())?;
+    } else {
+        if workspace_root.join(".git").exists() {
+            return Err(user_error_with_hint(
+                "Did not create a jj repo because there is an existing Git repo in this directory.",
+                format!(
+                    r#"To create a repo backed by the existing Git repo, run `jj git init --git-repo={}` instead."#,
+                    relative_wc_path.display()
+                ),
+            ));
+        }
+
+        Workspace::init_internal_git(command.settings(), workspace_root)?;
+    }
+
+    Ok(())
+}
+
+/// Imports branches and tags from the underlying Git repo, exports changes if
+/// the repo is colocated.
+///
+/// This is similar to `WorkspaceCommandHelper::import_git_refs()`, but never
+/// moves the Git HEAD to the working copy parent.
+fn init_git_refs(
+    ui: &mut Ui,
+    command: &CommandHelper,
+    repo: Arc<ReadonlyRepo>,
+    colocated: bool,
+) -> Result<Arc<ReadonlyRepo>, CommandError> {
+    let mut tx = start_repo_transaction(&repo, command.settings(), command.string_args());
+    // There should be no old refs to abandon, but enforce it.
+    let mut git_settings = command.settings().git_settings();
+    git_settings.abandon_unreachable_commits = false;
+    let stats = git::import_some_refs(
+        tx.mut_repo(),
+        &git_settings,
+        // Initial import shouldn't fail because of reserved remote name.
+        |ref_name| !git::is_reserved_git_remote_ref(ref_name),
+    )?;
+    if !tx.mut_repo().has_changes() {
+        return Ok(repo);
+    }
+    print_git_import_stats(ui, &stats)?;
+    if colocated {
+        // If git.auto-local-branch = true, local branches could be created for
+        // the imported remote branches.
+        let failed_branches = git::export_refs(tx.mut_repo())?;
+        print_failed_git_export(ui, &failed_branches)?;
+    }
+    let repo = tx.commit("import git refs");
+    writeln!(
+        ui.stderr(),
+        "Done importing changes from the underlying Git repo."
+    )?;
+    Ok(repo)
+}
+
+fn cmd_git_init(
+    ui: &mut Ui,
+    command: &CommandHelper,
+    args: &GitInitArgs,
+) -> Result<(), CommandError> {
+    let cwd = command.cwd().canonicalize().unwrap();
+    let wc_path = cwd.join(&args.destination);
+    let wc_path = file_util::create_or_reuse_dir(&wc_path)
+        .and_then(|_| wc_path.canonicalize())
+        .map_err(|e| user_error_with_message("Failed to create workspace", e))?;
+
+    git_init(
+        ui,
+        command,
+        &wc_path,
+        args.colocate,
+        args.git_repo.as_deref(),
+    )?;
+
+    let relative_wc_path = file_util::relative_path(&cwd, &wc_path);
+    writeln!(
+        ui.stderr(),
+        r#"Initialized repo in "{}""#,
+        relative_wc_path.display()
+    )?;
+
+    Ok(())
+}
+
 #[tracing::instrument(skip(ui, command))]
 fn cmd_git_fetch(
     ui: &mut Ui,
@@ -337,7 +508,7 @@ fn cmd_git_fetch(
     };
     let mut tx = workspace_command.start_transaction();
     for remote in &remotes {
-        let stats = with_remote_callbacks(ui, |cb| {
+        let stats = with_remote_git_callbacks(ui, |cb| {
             git::fetch(
                 tx.mut_repo(),
                 &git_repo,
@@ -355,16 +526,16 @@ fn cmd_git_fetch(
                     .any(|pattern| pattern.as_exact().map_or(false, |s| s.contains('*')))
                 {
                     user_error_with_hint(
-                        err.to_string(),
+                        err,
                         "Prefix the pattern with `glob:` to expand `*` as a glob",
                     )
                 } else {
-                    user_error(err.to_string())
+                    user_error(err)
                 }
             }
             GitFetchError::GitImportError(err) => err.into(),
             GitFetchError::InternalGitError(err) => map_git_error(err),
-            _ => user_error(err.to_string()),
+            _ => user_error(err),
         })?;
         print_git_import_stats(ui, &stats.import_stats)?;
     }
@@ -467,7 +638,12 @@ fn cmd_git_clone(
     let wc_path_existed = match fs::create_dir(&wc_path) {
         Ok(()) => false,
         Err(err) if err.kind() == io::ErrorKind::AlreadyExists => true,
-        Err(err) => return Err(user_error(format!("Failed to create {wc_path_str}: {err}"))),
+        Err(err) => {
+            return Err(user_error_with_message(
+                format!("Failed to create {wc_path_str}"),
+                err,
+            ));
+        }
     };
     if wc_path_existed && !is_empty_dir(&wc_path) {
         return Err(user_error(
@@ -479,7 +655,7 @@ fn cmd_git_clone(
     // `/some/path/.`
     let canonical_wc_path: PathBuf = wc_path
         .canonicalize()
-        .map_err(|err| user_error(format!("Failed to create {wc_path_str}: {err}")))?;
+        .map_err(|err| user_error_with_message(format!("Failed to create {wc_path_str}"), err))?;
     let clone_result = do_git_clone(
         ui,
         command,
@@ -555,7 +731,7 @@ fn do_git_clone(
     git_repo.remote(remote_name, source).unwrap();
     let mut fetch_tx = workspace_command.start_transaction();
 
-    let stats = with_remote_callbacks(ui, |cb| {
+    let stats = with_remote_git_callbacks(ui, |cb| {
         git::fetch(
             fetch_tx.mut_repo(),
             &git_repo,
@@ -580,119 +756,6 @@ fn do_git_clone(
     Ok((workspace_command, stats))
 }
 
-fn with_remote_callbacks<T>(ui: &mut Ui, f: impl FnOnce(git::RemoteCallbacks<'_>) -> T) -> T {
-    let mut ui = Mutex::new(ui);
-    let mut callback = None;
-    if let Some(mut output) = ui.get_mut().unwrap().progress_output() {
-        let mut progress = Progress::new(Instant::now());
-        callback = Some(move |x: &git::Progress| {
-            _ = progress.update(Instant::now(), x, &mut output);
-        });
-    }
-    let mut callbacks = git::RemoteCallbacks::default();
-    callbacks.progress = callback
-        .as_mut()
-        .map(|x| x as &mut dyn FnMut(&git::Progress));
-    let mut get_ssh_keys = get_ssh_keys; // Coerce to unit fn type
-    callbacks.get_ssh_keys = Some(&mut get_ssh_keys);
-    let mut get_pw = |url: &str, _username: &str| {
-        pinentry_get_pw(url).or_else(|| terminal_get_pw(*ui.lock().unwrap(), url))
-    };
-    callbacks.get_password = Some(&mut get_pw);
-    let mut get_user_pw = |url: &str| {
-        let ui = &mut *ui.lock().unwrap();
-        Some((terminal_get_username(ui, url)?, terminal_get_pw(ui, url)?))
-    };
-    callbacks.get_username_password = Some(&mut get_user_pw);
-    f(callbacks)
-}
-
-fn terminal_get_username(ui: &mut Ui, url: &str) -> Option<String> {
-    ui.prompt(&format!("Username for {url}")).ok()
-}
-
-fn terminal_get_pw(ui: &mut Ui, url: &str) -> Option<String> {
-    ui.prompt_password(&format!("Passphrase for {url}: ")).ok()
-}
-
-fn pinentry_get_pw(url: &str) -> Option<String> {
-    let mut pinentry = Command::new("pinentry")
-        .stdin(Stdio::piped())
-        .stdout(Stdio::piped())
-        .spawn()
-        .ok()?;
-    #[rustfmt::skip]
-    pinentry
-        .stdin
-        .take()
-        .unwrap()
-        .write_all(
-            format!(
-                "SETTITLE jj passphrase\n\
-                 SETDESC Enter passphrase for {url}\n\
-                 SETPROMPT Passphrase:\n\
-                 GETPIN\n"
-            )
-            .as_bytes(),
-        )
-        .ok()?;
-    let mut out = String::new();
-    pinentry
-        .stdout
-        .take()
-        .unwrap()
-        .read_to_string(&mut out)
-        .ok()?;
-    _ = pinentry.wait();
-    for line in out.split('\n') {
-        if !line.starts_with("D ") {
-            continue;
-        }
-        let (_, encoded) = line.split_at(2);
-        return decode_assuan_data(encoded);
-    }
-    None
-}
-
-// https://www.gnupg.org/documentation/manuals/assuan/Server-responses.html#Server-responses
-fn decode_assuan_data(encoded: &str) -> Option<String> {
-    let encoded = encoded.as_bytes();
-    let mut decoded = Vec::with_capacity(encoded.len());
-    let mut i = 0;
-    while i < encoded.len() {
-        if encoded[i] != b'%' {
-            decoded.push(encoded[i]);
-            i += 1;
-            continue;
-        }
-        i += 1;
-        let byte =
-            u8::from_str_radix(std::str::from_utf8(encoded.get(i..i + 2)?).ok()?, 16).ok()?;
-        decoded.push(byte);
-        i += 2;
-    }
-    String::from_utf8(decoded).ok()
-}
-
-#[tracing::instrument]
-fn get_ssh_keys(_username: &str) -> Vec<PathBuf> {
-    let mut paths = vec![];
-    if let Some(home_dir) = dirs::home_dir() {
-        let ssh_dir = Path::new(&home_dir).join(".ssh");
-        for filename in ["id_ed25519_sk", "id_ed25519", "id_rsa"] {
-            let key_path = ssh_dir.join(filename);
-            if key_path.is_file() {
-                tracing::info!(path = ?key_path, "found ssh key");
-                paths.push(key_path);
-            }
-        }
-    }
-    if paths.is_empty() {
-        tracing::info!("no ssh key found");
-    }
-    paths
-}
-
 fn cmd_git_push(
     ui: &mut Ui,
     command: &CommandHelper,
@@ -727,6 +790,18 @@ fn cmd_git_push(
             }
         }
         tx_description = format!("push all branches to git remote {remote}");
+    } else if args.tracked {
+        for (branch_name, targets) in repo.view().local_remote_branches(&remote) {
+            if !targets.remote_ref.is_tracking() {
+                continue;
+            }
+            match classify_branch_update(branch_name, &remote, targets) {
+                Ok(Some(update)) => branch_updates.push((branch_name.to_owned(), update)),
+                Ok(None) => {}
+                Err(reason) => reason.print(ui)?,
+            }
+        }
+        tx_description = format!("push all tracked branches to git remote {remote}");
     } else if args.deleted {
         for (branch_name, targets) in repo.view().local_remote_branches(&remote) {
             if targets.local_target.is_present() {
@@ -847,11 +922,20 @@ fn cmd_git_push(
                 Err(reason) => reason.print(ui)?,
             }
         }
-        if (!args.revisions.is_empty() || use_default_revset) && branches_targeted.is_empty() {
-            writeln!(
-                ui.warning(),
-                "No branches point to the specified revisions."
-            )?;
+        if branches_targeted.is_empty() {
+            if use_default_revset {
+                writeln!(
+                    ui.warning(),
+                    "No branches found in the default push revset, \
+                     `remote_branches(remote={remote})..@`."
+                )?;
+            } else if !args.revisions.is_empty() {
+                writeln!(
+                    ui.warning(),
+                    "No branches point to the specified revisions."
+                )?;
+            } else { /* A plain "Nothing changed" message will suffice */
+            }
         }
 
         tx_description = format!(
@@ -977,7 +1061,7 @@ fn cmd_git_push(
         branch_updates,
         force_pushed_branches,
     };
-    with_remote_callbacks(ui, |cb| {
+    with_remote_git_callbacks(ui, |cb| {
         git::push_branches(tx.mut_repo(), &git_repo, &remote, &targets, cb)
     })
     .map_err(|err| match err {
@@ -987,7 +1071,7 @@ fn cmd_git_push(
             "Try fetching from the remote, then make the branch point to where you want it to be, \
              and push again.",
         ),
-        _ => user_error(err.to_string()),
+        _ => user_error(err),
     })?;
     tx.finish(ui, tx_description)?;
     Ok(())
@@ -1030,7 +1114,7 @@ impl RejectedBranchUpdateReason {
 impl From<RejectedBranchUpdateReason> for CommandError {
     fn from(reason: RejectedBranchUpdateReason) -> Self {
         let RejectedBranchUpdateReason { message, hint } = reason;
-        CommandError::UserError { message, hint }
+        user_error_with_hint_opt(message, hint)
     }
 }
 
@@ -1102,6 +1186,9 @@ fn cmd_git_import(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let mut tx = workspace_command.start_transaction();
+    // In non-colocated repo, HEAD@git will never be moved internally by jj.
+    // That's why cmd_git_export() doesn't export the HEAD ref.
+    git::import_head(tx.mut_repo())?;
     let stats = git::import_refs(tx.mut_repo(), &command.settings().git_settings())?;
     print_git_import_stats(ui, &stats)?;
     tx.finish(ui, "import git refs")?;
@@ -1161,6 +1248,7 @@ pub fn cmd_git(
     subcommand: &GitCommand,
 ) -> Result<(), CommandError> {
     match subcommand {
+        GitCommand::Init(args) => cmd_git_init(ui, command, args),
         GitCommand::Fetch(args) => cmd_git_fetch(ui, command, args),
         GitCommand::Clone(args) => cmd_git_clone(ui, command, args),
         GitCommand::Remote(GitRemoteCommand::Add(args)) => cmd_git_remote_add(ui, command, args),
