@@ -24,6 +24,14 @@ pub(crate) fn cmd_merge(
     command: &CommandHelper,
     args: &new::NewArgs,
 ) -> Result<(), CommandError> {
+    writeln!(
+        ui.warning(),
+        "warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent"
+    )?;
+    writeln!(
+        ui.warning(),
+        "warning: `jj merge` will be removed in a future version, and this will be a hard error"
+    )?;
     if args.revisions.len() < 2 {
         return Err(CommandError::CliError(String::from(
             "Merge requires at least two revisions",
