@@ -43,12 +43,14 @@ mod prev;
 mod rebase;
 mod resolve;
 mod restore;
+mod root;
 mod run;
 mod show;
 mod sparse;
 mod split;
 mod squash;
 mod status;
+mod tag;
 mod unsquash;
 mod untrack;
 mod util;
@@ -71,14 +73,15 @@ enum Command {
     #[command(subcommand)]
     Bench(bench::BenchCommand),
     #[command(subcommand)]
-    Branch(branch::BranchSubcommand),
+    Branch(branch::BranchCommand),
     #[command(alias = "print")]
     Cat(cat::CatArgs),
+    #[command(hide = true)]
     Checkout(checkout::CheckoutArgs),
     Chmod(chmod::ChmodArgs),
     Commit(commit::CommitArgs),
     #[command(subcommand)]
-    Config(config::ConfigSubcommand),
+    Config(config::ConfigCommand),
     #[command(subcommand)]
     Debug(debug::DebugCommand),
     Describe(describe::DescribeArgs),
@@ -101,6 +104,7 @@ enum Command {
     ///
     /// This is the same as `jj new`, except that it requires at least two
     /// arguments.
+    #[command(hide = true)]
     Merge(new::NewArgs),
     Move(r#move::MoveArgs),
     New(new::NewArgs),
@@ -118,6 +122,7 @@ enum Command {
         help_template = "Not a real subcommand; consider `jj backout` or `jj restore`"
     )]
     Revert(DummyCommandArgs),
+    Root(root::RootArgs),
     #[command(hide = true)]
     // TODO: Flesh out.
     Run(run::RunArgs),
@@ -128,6 +133,8 @@ enum Command {
     Squash(squash::SquashArgs),
     Status(status::StatusArgs),
     #[command(subcommand)]
+    Tag(tag::TagCommand),
+    #[command(subcommand)]
     Util(util::UtilCommand),
     /// Undo an operation (shortcut for `jj op undo`)
     Undo(operation::OperationUndoArgs),
@@ -179,6 +186,7 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Unsquash(sub_args) => unsquash::cmd_unsquash(ui, command_helper, sub_args),
         Command::Restore(sub_args) => restore::cmd_restore(ui, command_helper, sub_args),
         Command::Revert(_args) => revert(),
+        Command::Root(sub_args) => root::cmd_root(ui, command_helper, sub_args),
         Command::Run(sub_args) => run::cmd_run(ui, command_helper, sub_args),
         Command::Diffedit(sub_args) => diffedit::cmd_diffedit(ui, command_helper, sub_args),
         Command::Split(sub_args) => split::cmd_split(ui, command_helper, sub_args),
@@ -191,6 +199,7 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Operation(sub_args) => operation::cmd_operation(ui, command_helper, sub_args),
         Command::Workspace(sub_args) => workspace::cmd_workspace(ui, command_helper, sub_args),
         Command::Sparse(sub_args) => sparse::cmd_sparse(ui, command_helper, sub_args),
+        Command::Tag(sub_args) => tag::cmd_tag(ui, command_helper, sub_args),
         Command::Chmod(sub_args) => chmod::cmd_chmod(ui, command_helper, sub_args),
         Command::Git(sub_args) => git::cmd_git(ui, command_helper, sub_args),
         Command::Util(sub_args) => util::cmd_util(ui, command_helper, sub_args),
