@@ -13,7 +13,7 @@
 // limitations under the License.
 
 use clap::ArgGroup;
-use jj_lib::backend::ObjectId;
+use jj_lib::object_id::ObjectId;
 use jj_lib::repo::Repo;
 use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
