@@ -19,8 +19,9 @@ use std::sync::Arc;
 use clap::ArgGroup;
 use indexmap::IndexSet;
 use itertools::Itertools;
-use jj_lib::backend::{CommitId, ObjectId};
+use jj_lib::backend::CommitId;
 use jj_lib::commit::Commit;
+use jj_lib::object_id::ObjectId;
 use jj_lib::repo::{ReadonlyRepo, Repo};
 use jj_lib::revset::{RevsetExpression, RevsetIteratorExt};
 use jj_lib::rewrite::{rebase_commit, rebase_commit_with_options, EmptyBehaviour, RebaseOptions};
@@ -45,6 +46,7 @@ use crate::ui::Ui;
 /// your history like this (letters followed by an apostrophe are post-rebase
 /// versions):
 ///
+/// ```text
 /// O           N'
 /// |           |
 /// | N         M'
@@ -56,6 +58,7 @@ use crate::ui::Ui;
 /// | K         | K
 /// |/          |/
 /// J           J
+/// ```
 ///
 /// With `-b`, the command rebases the whole "branch" containing the specified
 /// revision. A "branch" is the set of commits that includes:
@@ -70,6 +73,7 @@ use crate::ui::Ui;
 /// -d O` would transform your history like this (because `L` and `M` are on the
 /// same "branch", relative to the destination):
 ///
+/// ```text
 /// O           N'
 /// |           |
 /// | N         M'
@@ -81,12 +85,14 @@ use crate::ui::Ui;
 /// | K         O
 /// |/          |
 /// J           J
+/// ```
 ///
 /// With `-r`, the command rebases only the specified revision onto the
 /// destination. Any "hole" left behind will be filled by rebasing descendants
 /// onto the specified revision's parent(s). For example, `jj rebase -r K -d M`
 /// would transform your history like this:
 ///
+/// ```text
 /// M          K'
 /// |          |
 /// | L        M
@@ -94,12 +100,14 @@ use crate::ui::Ui;
 /// | K        | L'
 /// |/         |/
 /// J          J
+/// ```
 ///
 /// Note that you can create a merge commit by repeating the `-d` argument.
 /// For example, if you realize that commit L actually depends on commit M in
 /// order to work (in addition to its current parent K), you can run `jj rebase
 /// -s L -d K -d M`:
 ///
+/// ```text
 /// M          L'
 /// |          |\
 /// | L        M |
@@ -107,6 +115,7 @@ use crate::ui::Ui;
 /// | K        | K
 /// |/         |/
 /// J          J
+/// ```
 #[derive(clap::Args, Clone, Debug)]
 #[command(verbatim_doc_comment)]
 #[command(group(ArgGroup::new("to_rebase").args(&["branch", "source", "revision"])))]
@@ -148,6 +157,7 @@ pub(crate) struct RebaseArgs {
     /// If true, when rebasing would produce an empty commit, the commit is
     /// skipped.
     /// Will never skip merge commits with multiple non-empty parents.
+    /// Will never skip the working commit.
     #[arg(long, conflicts_with = "revision")]
     skip_empty: bool,
 
