@@ -16,8 +16,9 @@ use std::collections::BTreeMap;
 use std::io::Write;
 
 use itertools::Itertools;
-use jj_lib::backend::{ObjectId, TreeValue};
+use jj_lib::backend::TreeValue;
 use jj_lib::merge::MergedTreeValue;
+use jj_lib::object_id::ObjectId;
 use jj_lib::repo_path::RepoPathBuf;
 use tracing::instrument;
 
@@ -95,6 +96,11 @@ pub(crate) fn cmd_resolve(
 
     let (repo_path, _) = conflicts.first().unwrap();
     workspace_command.check_rewritable([&commit])?;
+    writeln!(
+        ui.stderr(),
+        "Resolving conflicts in: {}",
+        workspace_command.format_file_path(repo_path)
+    )?;
     let mut tx = workspace_command.start_transaction();
     let new_tree_id = tx.run_mergetool(ui, &tree, repo_path)?;
     let new_commit = tx
