@@ -4,12 +4,13 @@ use std::sync::Arc;
 
 use futures::{StreamExt, TryStreamExt};
 use itertools::Itertools;
-use jj_lib::backend::{BackendError, FileId, MergedTreeId, ObjectId, TreeValue};
+use jj_lib::backend::{BackendError, FileId, MergedTreeId, TreeValue};
 use jj_lib::diff::{find_line_ranges, Diff, DiffHunk};
 use jj_lib::files::{self, ContentHunk, MergeResult};
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::Merge;
 use jj_lib::merged_tree::{MergedTree, MergedTreeBuilder};
+use jj_lib::object_id::ObjectId;
 use jj_lib::repo_path::{RepoPath, RepoPathBuf};
 use jj_lib::store::Store;
 use pollster::FutureExt;
@@ -17,11 +18,11 @@ use thiserror::Error;
 
 #[derive(Debug, Error)]
 pub enum BuiltinToolError {
-    #[error("Failed to record changes: {0}")]
+    #[error("Failed to record changes")]
     Record(#[from] scm_record::RecordError),
     #[error(transparent)]
     ReadFileBackend(BackendError),
-    #[error("Failed to read file {path:?} with ID {id}: {source}", id = id.hex())]
+    #[error("Failed to read file {path:?} with ID {id}", id = id.hex())]
     ReadFileIo {
         path: RepoPathBuf,
         id: FileId,
@@ -29,14 +30,14 @@ pub enum BuiltinToolError {
     },
     #[error(transparent)]
     ReadSymlink(BackendError),
-    #[error("Failed to decode UTF-8 text for item {item} (this should not happen): {source}")]
+    #[error("Failed to decode UTF-8 text for item {item} (this should not happen)")]
     DecodeUtf8 {
         source: std::str::Utf8Error,
         item: &'static str,
     },
     #[error("Rendering {item} {id} is unimplemented for the builtin difftool/mergetool")]
     Unimplemented { item: &'static str, id: String },
-    #[error("Backend error: {0}")]
+    #[error("Backend error")]
     BackendError(#[from] jj_lib::backend::BackendError),
 }
 
