@@ -16,8 +16,6 @@ use std::path::Path;
 
 use crate::common::TestEnvironment;
 
-pub mod common;
-
 #[test]
 fn test_branch_multiple_names() {
     let test_env = TestEnvironment::default();
@@ -195,20 +193,42 @@ fn test_branch_rename() {
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
 
-    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "rename", "foo", "bar"]);
+    // Set up remote
+    let git_repo_path = test_env.env_root().join("git-repo");
+    git2::Repository::init_bare(git_repo_path).unwrap();
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &["git", "remote", "add", "origin", "../git-repo"],
+    );
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "rename", "bnoexist", "blocal"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: No such branch: foo
+    Error: No such branch: bnoexist
     "###);
 
-    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo"]);
-    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "rename", "foo", "bar"]);
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=commit-0"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "blocal"]);
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["branch", "rename", "blocal", "blocal1"]);
     insta::assert_snapshot!(stderr, @"");
 
     test_env.jj_cmd_ok(&repo_path, &["new"]);
-    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "conflictfoo"]);
-    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "rename", "bar", "conflictfoo"]);
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=commit-1"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "bexist"]);
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "rename", "blocal1", "bexist"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Branch already exists: conflictfoo
+    Error: Branch already exists: bexist
+    "###);
+
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=commit-2"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "bremote"]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "push", "-b=bremote"]);
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["branch", "rename", "bremote", "bremote2"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Warning: Branch bremote has tracking remote branches which were not renamed.
+    Hint: to rename the branch on the remote, you can `jj git push --branch bremote` first (to delete it on the remote), and then `jj git push --branch bremote2`. `jj git push --all` would also be sufficient.
     "###);
 }
 
@@ -1118,6 +1138,50 @@ fn test_branch_list_filtered() {
     "###);
 }
 
+#[test]
+fn test_branch_list_much_remote_divergence() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
+
+    // Initialize remote refs
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "remote", "--git"]);
+    let remote_path = test_env.env_root().join("remote");
+    test_env.jj_cmd_ok(&remote_path, &["new", "root()", "-m", "remote-unsync"]);
+    for _ in 0..15 {
+        test_env.jj_cmd_ok(&remote_path, &["new", "-m", "remote-unsync"]);
+    }
+    test_env.jj_cmd_ok(&remote_path, &["branch", "create", "remote-unsync"]);
+    test_env.jj_cmd_ok(&remote_path, &["new"]);
+    test_env.jj_cmd_ok(&remote_path, &["git", "export"]);
+
+    // Initialize local refs
+    let mut remote_git_path = remote_path;
+    remote_git_path.extend([".jj", "repo", "store", "git"]);
+    test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &["git", "clone", remote_git_path.to_str().unwrap(), "local"],
+    );
+    let local_path = test_env.env_root().join("local");
+    test_env.jj_cmd_ok(&local_path, &["new", "root()", "-m", "local-only"]);
+    for _ in 0..15 {
+        test_env.jj_cmd_ok(&local_path, &["new", "-m", "local-only"]);
+    }
+    test_env.jj_cmd_ok(&local_path, &["branch", "create", "local-only"]);
+
+    // Mutate refs in local repository
+    test_env.jj_cmd_ok(
+        &local_path,
+        &["branch", "set", "--allow-backwards", "remote-unsync"],
+    );
+
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&local_path, &["branch", "list"]), @r###"
+    local-only: zkyosouw 4ab3f751 (empty) local-only
+    remote-unsync: zkyosouw 4ab3f751 (empty) local-only
+      @origin (ahead by at least 10 commits, behind by at least 10 commits): lxyktnks 19582022 (empty) remote-unsync
+    "###);
+}
+
 fn get_log_output(test_env: &TestEnvironment, cwd: &Path) -> String {
     let template = r#"branches ++ " " ++ commit_id.short()"#;
     test_env.jj_cmd_success(cwd, &["log", "-T", template])
