@@ -14,8 +14,6 @@
 
 use crate::common::{escaped_fake_diff_editor_path, TestEnvironment};
 
-pub mod common;
-
 #[test]
 fn test_diffedit() {
     let mut test_env = TestEnvironment::default();
@@ -45,18 +43,19 @@ fn test_diffedit() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     "###);
 
     // Nothing happens if the diff-editor exits with an error
     std::fs::write(&edit_script, "rm file2\0fail").unwrap();
     insta::assert_snapshot!(&test_env.jj_cmd_failure(&repo_path, &["diffedit"]), @r###"
-    Error: Failed to edit diff: Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
+    Error: Failed to edit diff
+    Caused by: Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     "###);
 
@@ -72,7 +71,7 @@ fn test_diffedit() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     "###);
 
     // Changes to a commit are propagated to descendants
@@ -109,8 +108,8 @@ fn test_diffedit() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
-    R file2
+    D file1
+    D file2
     "###);
 }
 
@@ -140,7 +139,7 @@ fn test_diffedit_new_file() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     A file2
     "###);
 
@@ -175,7 +174,7 @@ fn test_diffedit_new_file() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     A file2
     "###);
 }
@@ -224,7 +223,7 @@ fn test_diffedit_3pane() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     "###);
     // Nothing happens if we make no changes, `config_with_right_as_after` version
@@ -238,7 +237,7 @@ fn test_diffedit_3pane() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     "###);
 
@@ -257,7 +256,7 @@ fn test_diffedit_3pane() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     "###);
 
     // Can write something new to `file1`
@@ -293,7 +292,7 @@ fn test_diffedit_3pane() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     "###);
 
@@ -348,7 +347,7 @@ fn test_diffedit_merge() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "-r", "@-"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     A file3
     "###);
     assert!(!repo_path.join("file1").exists());
@@ -387,7 +386,7 @@ fn test_diffedit_old_restore_interactive_tests() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     A file3
     "###);
@@ -395,11 +394,12 @@ fn test_diffedit_old_restore_interactive_tests() {
     // Nothing happens if the diff-editor exits with an error
     std::fs::write(&edit_script, "rm file2\0fail").unwrap();
     insta::assert_snapshot!(&test_env.jj_cmd_failure(&repo_path, &["diffedit", "--from", "@-"]), @r###"
-    Error: Failed to edit diff: Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
+    Error: Failed to edit diff
+    Caused by: Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     A file3
     "###);
@@ -416,7 +416,7 @@ fn test_diffedit_old_restore_interactive_tests() {
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     "###);
 
     // Can make unrelated edits
