@@ -16,8 +16,6 @@ use std::ffi::OsString;
 
 use crate::common::{get_stderr_string, TestEnvironment};
 
-pub mod common;
-
 #[test]
 fn test_non_utf8_arg() {
     let test_env = TestEnvironment::default();
@@ -42,6 +40,22 @@ fn test_non_utf8_arg() {
     "###);
 }
 
+#[test]
+fn test_version() {
+    let test_env = TestEnvironment::default();
+
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["--version"]);
+    let sanitized = stdout.replace(|c: char| c.is_ascii_hexdigit(), "?");
+    let expected = [
+        "jj ?.??.?\n",
+        "jj ?.??.?-????????????????????????????????????????\n",
+    ];
+    assert!(
+        expected.contains(&sanitized.as_str()),
+        "`jj version` output: {stdout:?}.\nSanitized: {sanitized:?}\nExpected one of: {expected:?}"
+    );
+}
+
 #[test]
 fn test_no_subcommand() {
     let test_env = TestEnvironment::default();
@@ -52,7 +66,7 @@ fn test_no_subcommand() {
     let stderr = test_env.jj_cmd_failure(test_env.env_root(), &[]);
     insta::assert_snapshot!(stderr, @r###"
     Hint: Use `jj -h` for a list of available commands.
-    Set the config `ui.default-command = "log"` to disable this message.
+    Run `jj config set --user ui.default-command log` to disable this message.
     Error: There is no jj repo in "."
     "###);
 
@@ -65,14 +79,6 @@ fn test_no_subcommand() {
     let stdout = test_env.jj_cmd_success(test_env.env_root(), &["--help"]);
     insta::assert_snapshot!(stdout.lines().next().unwrap(), @"Jujutsu (An experimental VCS)");
 
-    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["--version"]);
-    let sanitized = stdout.replace(|c: char| c.is_ascii_hexdigit(), "?");
-    assert!(
-        sanitized == "jj ?.??.?\n"
-            || sanitized == "jj ?.??.?-????????????????????????????????????????\n",
-        "{sanitized}"
-    );
-
     let stdout = test_env.jj_cmd_success(test_env.env_root(), &["-R", "repo"]);
     assert_eq!(stdout, test_env.jj_cmd_success(&repo_path, &["log"]));
 
@@ -204,7 +210,7 @@ fn test_no_workspace_directory() {
     insta::assert_snapshot!(stderr, @r###"
     Error: There is no jj repo in "."
     Hint: It looks like this is a git repo. You can create a jj repo backed by it by running this:
-    jj init --git-repo=.
+    jj git init --git-repo=.
     "###);
 }
 
@@ -219,15 +225,18 @@ fn test_broken_repo_structure() {
     // Test the error message when the git repository can't be located.
     std::fs::remove_file(store_path.join("git_target")).unwrap();
     let stderr = test_env.jj_cmd_internal_error(&repo_path, &["log"]);
-    insta::assert_snapshot!(stderr, @r###"
-    Internal error: The repository appears broken or inaccessible: Cannot access $TEST_ENV/repo/.jj/repo/store/git_target
+    insta::assert_snapshot!(strip_last_line(&stderr), @r###"
+    Internal error: The repository appears broken or inaccessible
+    Caused by:
+    1: Cannot access $TEST_ENV/repo/.jj/repo/store/git_target
     "###);
 
     // Test the error message when the commit backend is of unknown type.
     std::fs::write(&store_type_path, "unknown").unwrap();
     let stderr = test_env.jj_cmd_internal_error(&repo_path, &["log"]);
     insta::assert_snapshot!(stderr, @r###"
-    Internal error: This version of the jj binary doesn't support this type of repo: Unsupported commit backend type 'unknown'
+    Internal error: This version of the jj binary doesn't support this type of repo
+    Caused by: Unsupported commit backend type 'unknown'
     "###);
 
     // Test the error message when the file indicating the commit backend type
@@ -235,8 +244,11 @@ fn test_broken_repo_structure() {
     std::fs::remove_file(&store_type_path).unwrap();
     std::fs::create_dir(&store_type_path).unwrap();
     let stderr = test_env.jj_cmd_internal_error(&repo_path, &["log"]);
-    insta::assert_snapshot!(stderr, @r###"
-    Internal error: The repository appears broken or inaccessible: Failed to read commit backend type: Cannot access $TEST_ENV/repo/.jj/repo/store/type
+    insta::assert_snapshot!(strip_last_line(&stderr), @r###"
+    Internal error: The repository appears broken or inaccessible
+    Caused by:
+    1: Failed to read commit backend type
+    2: Cannot access $TEST_ENV/repo/.jj/repo/store/type
     "###);
 
     // Test when the .jj directory is empty. The error message is identical to
@@ -244,8 +256,11 @@ fn test_broken_repo_structure() {
     std::fs::remove_dir_all(repo_path.join(".jj")).unwrap();
     std::fs::create_dir(repo_path.join(".jj")).unwrap();
     let stderr = test_env.jj_cmd_internal_error(&repo_path, &["log"]);
-    insta::assert_snapshot!(stderr, @r###"
-    Internal error: The repository appears broken or inaccessible: Failed to read commit backend type: Cannot access $TEST_ENV/repo/.jj/repo/store/type
+    insta::assert_snapshot!(strip_last_line(&stderr), @r###"
+    Internal error: The repository appears broken or inaccessible
+    Caused by:
+    1: Failed to read commit backend type
+    2: Cannot access $TEST_ENV/repo/.jj/repo/store/type
     "###);
 }
 
@@ -456,3 +471,9 @@ fn test_verbose_logging_enabled() {
     // Luckily, insta will print this in colour when reviewing.
     insta::assert_snapshot!(log_line, @"[32m INFO[0m [2mjj_cli::cli_util[0m[2m:[0m verbose logging enabled");
 }
+
+fn strip_last_line(s: &str) -> &str {
+    s.trim_end_matches('\n')
+        .rsplit_once('\n')
+        .map_or(s, |(h, _)| &s[..h.len() + 1])
+}
