@@ -18,8 +18,6 @@ use test_case::test_case;
 
 use crate::common::TestEnvironment;
 
-pub mod common;
-
 fn init_git_repo(git_repo_path: &Path, bare: bool) -> git2::Repository {
     init_git_repo_with_opts(git_repo_path, git2::RepositoryInitOptions::new().bare(bare))
 }
@@ -74,6 +72,8 @@ fn test_init_git_internal() {
     let (stdout, stderr) = test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "repo"
     "###);
 
@@ -109,9 +109,12 @@ fn test_init_git_external(bare: bool) {
     insta::allow_duplicates! {
         insta::assert_snapshot!(stdout, @"");
         insta::assert_snapshot!(stderr, @r###"
+        Done importing changes from the underlying Git repo.
         Working copy now at: sqpuoqvx f6950fc1 (empty) (no description set)
         Parent commit      : mwrttmos 8d698d4a my-branch | My commit message
         Added 1 files, modified 0 files, removed 0 files
+        warning: `--git` and `--git-repo` are deprecated.
+        Use `jj git init` instead
         Initialized repo in "repo"
         "###);
     }
@@ -162,11 +165,13 @@ fn test_init_git_external_non_existent_git_directory() {
     let stderr =
         test_env.jj_cmd_failure(test_env.env_root(), &["init", "repo", "--git-repo", "repo"]);
 
-    insta::with_settings!({filters => vec![
-        (r"(Failed to open git repository:)(?s).*", "Failed to open git repository:"),
-    ]}, {
-        insta::assert_snapshot!(&stderr, @"Error: Failed to access the repository: Failed to open git repository:");
-    });
+    insta::assert_snapshot!(&stderr, @r###"
+    Error: Failed to access the repository
+    Caused by:
+    1: Failed to open git repository
+    2: "$TEST_ENV/repo" does not appear to be a git repository
+    3: Missing HEAD at '.git/HEAD'
+    "###);
     let jj_path = workspace_root.join(".jj");
     assert!(!jj_path.exists());
 }
@@ -180,6 +185,8 @@ fn test_init_git_colocated() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
 
@@ -228,6 +235,8 @@ fn test_init_git_colocated_gitlink() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
@@ -264,6 +273,8 @@ fn test_init_git_colocated_symlink_directory() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
@@ -303,6 +314,8 @@ fn test_init_git_colocated_symlink_directory_without_bare_config() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
@@ -344,6 +357,8 @@ fn test_init_git_colocated_symlink_gitlink() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
@@ -404,6 +419,8 @@ fn test_init_git_colocated_imported_refs() {
     let (_stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["init", "--git-repo=."]);
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &local_path), @r###"
@@ -425,6 +442,8 @@ fn test_init_git_colocated_imported_refs() {
     The following remote branches aren't associated with the existing local branches:
       local-remote@origin
     Hint: Run `jj branch track local-remote@origin` to keep local branches updated on future pulls.
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &local_path), @r###"
@@ -448,6 +467,8 @@ fn test_init_git_external_but_git_dir_exists() {
     );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    warning: `--git` and `--git-repo` are deprecated.
+    Use `jj git init` instead
     Initialized repo in "."
     "###);
 
@@ -476,7 +497,7 @@ fn test_init_git_internal_must_be_colocated() {
     let stderr = test_env.jj_cmd_failure(&workspace_root, &["init", "--git"]);
     insta::assert_snapshot!(stderr, @r###"
     Error: Did not create a jj repo because there is an existing Git repo in this directory.
-    Hint: To create a repo backed by the existing Git repo, run `jj init --git-repo=.` instead.
+    Hint: To create a repo backed by the existing Git repo, run `jj git init --git-repo=.` instead.
     "###);
 }
 
@@ -494,7 +515,7 @@ fn test_init_local_disallowed() {
     let stdout = test_env.jj_cmd_failure(test_env.env_root(), &["init", "repo"]);
     insta::assert_snapshot!(stdout, @r###"
     Error: The native backend is disallowed by default.
-    Hint: Did you mean to pass `--git`?
+    Hint: Did you mean to call `jj git init`?
     Set `ui.allow-init-native` to allow initializing a repo with the native backend.
     "###);
 }
