@@ -13,9 +13,7 @@
 // limitations under the License.
 //
 
-use crate::common::TestEnvironment;
-
-pub mod common;
+use crate::common::{get_stderr_string, get_stdout_string, TestEnvironment};
 
 #[test]
 fn test_next_simple() {
@@ -117,23 +115,16 @@ fn test_next_exceeding_history() {
 }
 
 #[test]
-fn test_next_fails_on_branching_children() {
-    // TODO(#2126): Fix this behavior
+fn test_next_fails_on_merge_commit() {
     let test_env = TestEnvironment::default();
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
-    // Create a main branch for this test
-    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "left"]);
     test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "first"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "right"]);
     test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "second"]);
-    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "third"]);
-    // Create a branching child.
-    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "into-the-future"]);
-    test_env.jj_cmd_ok(&repo_path, &["co", "into-the-future"]);
-    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "42"]);
-    test_env.jj_cmd_ok(&repo_path, &["co", "main"]);
-    // Make the default branch have two possible children.
-    test_env.jj_cmd_ok(&repo_path, &["new", "into-the-future", "@-"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "left", "right"]);
     // Try to advance the working copy commit.
     let stderr = test_env.jj_cmd_failure(&repo_path, &["next"]);
     insta::assert_snapshot!(stderr,@r###"
@@ -142,26 +133,125 @@ fn test_next_fails_on_branching_children() {
 }
 
 #[test]
-fn test_prev_fails_on_multiple_parents() {
-    // TODO(#2126): Fix this behavior
+fn test_next_fails_on_branching_children_no_stdin() {
     let test_env = TestEnvironment::default();
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "first"]);
     test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "second"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
     test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "third"]);
-    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "all-about"]);
-    test_env.jj_cmd_ok(&repo_path, &["co", "all-about"]);
-    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "soname"]);
-    // Create a merge commit, which has two parents.
-    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "@--"]);
-    // We have more than one parent, prev fails.
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+
+    // Try to advance the working copy commit.
+    let assert = test_env.jj_cmd(&repo_path, &["next"]).assert().code(1);
+    let stderr = test_env.normalize_output(&get_stderr_string(&assert));
+    insta::assert_snapshot!(stderr,@r###"
+    Error: Cannot prompt for input since the output is not connected to a terminal
+    "###);
+}
+
+#[test]
+fn test_next_fails_on_branching_children_quit_prompt() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "first"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "second"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "third"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+
+    // Try to advance the working copy commit.
+    let assert = test_env
+        .jj_cmd_stdin(&repo_path, &["next"], "q\n")
+        .assert()
+        .code(1);
+    let stdout = test_env.normalize_output(&get_stdout_string(&assert));
+    let stderr = test_env.normalize_output(&get_stderr_string(&assert));
+    insta::assert_snapshot!(stdout,@r###"
+    ambiguous next commit, choose one to target:
+    1: zsuskuln 40a959a0 (empty) third
+    2: rlvkpnrz 5c52832c (empty) second
+    q: quit the prompt
+    enter the index of the commit you want to target: 
+    "###);
+    insta::assert_snapshot!(stderr,@r###"
+    Error: ambiguous target commit
+    "###);
+}
+
+#[test]
+fn test_next_choose_branching_child() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "first"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "second"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "third"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+    // Advance the working copy commit.
+    let (stdout, stderr) = test_env.jj_cmd_stdin_ok(&repo_path, &["next"], "1\n");
+    insta::assert_snapshot!(stdout,@r###"
+    ambiguous next commit, choose one to target:
+    1: zsuskuln 40a959a0 (empty) third
+    2: rlvkpnrz 5c52832c (empty) second
+    q: quit the prompt
+    enter the index of the commit you want to target: 
+    "###);
+    insta::assert_snapshot!(stderr,@r###"
+    Working copy now at: yqosqzyt f9f7101a (empty) (no description set)
+    Parent commit      : zsuskuln 40a959a0 (empty) third
+    "###);
+}
+
+#[test]
+fn test_prev_fails_on_merge_commit() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "left"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "first"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "right"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "second"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "left", "right"]);
+    // Try to advance the working copy commit.
     let stderr = test_env.jj_cmd_failure(&repo_path, &["prev"]);
     insta::assert_snapshot!(stderr,@r###"
     Error: Cannot run `jj prev` on a merge commit
     "###);
 }
 
+#[test]
+fn test_prev_fails_on_multiple_parents() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "left"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "first"]);
+    test_env.jj_cmd_ok(&repo_path, &["co", "@--"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "c", "right"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "second"]);
+    // Create a merge commit, which has two parents.
+    test_env.jj_cmd_ok(&repo_path, &["new", "left", "right"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "merge"]);
+    // Advance the working copy commit.
+    let (stdout, stderr) = test_env.jj_cmd_stdin_ok(&repo_path, &["prev"], "2\n");
+    insta::assert_snapshot!(stdout,@r###"
+    ambiguous prev commit, choose one to target:
+    1: zsuskuln edad76e9 right | (empty) second
+    2: qpvuntsm 5ae1a6a5 left | (empty) first
+    q: quit the prompt
+    enter the index of the commit you want to target: 
+    "###);
+    insta::assert_snapshot!(stderr,@r###"
+    Working copy now at: yostqsxw a9de0711 (empty) (no description set)
+    Parent commit      : qpvuntsm 5ae1a6a5 left | (empty) first
+    "###);
+}
+
 #[test]
 fn test_prev_onto_root_fails() {
     let test_env = TestEnvironment::default();
