@@ -19,6 +19,8 @@ mod entry;
 mod mutable;
 mod readonly;
 mod rev_walk;
+pub mod revset_engine;
+pub mod revset_graph_iterator;
 mod store;
 
 pub use self::composite::{AsCompositeIndex, CompositeIndex, IndexLevelStats, IndexStats};
@@ -43,8 +45,9 @@ mod tests {
     use super::entry::SmallIndexPositionsVec;
     use super::mutable::MutableIndexSegment;
     use super::*;
-    use crate::backend::{ChangeId, CommitId, ObjectId};
-    use crate::index::{HexPrefix, Index, PrefixResolution};
+    use crate::backend::{ChangeId, CommitId};
+    use crate::index::Index;
+    use crate::object_id::{HexPrefix, ObjectId, PrefixResolution};
 
     /// Generator of unique 16-byte ChangeId excluding root id
     fn change_id_generator() -> impl FnMut() -> ChangeId {
@@ -929,7 +932,7 @@ mod tests {
         let mut index = DefaultMutableIndex::full(3, 16);
         // Long linear history with some short branches
         let ids = (0..11)
-            .map(|n| CommitId::from_hex(&format!("{n:06x}")))
+            .map(|n| CommitId::try_from_hex(&format!("{n:06x}")).unwrap())
             .collect_vec();
         index.add_commit_data(ids[0].clone(), new_change_id(), &[]);
         for i in 1..ids.len() {
@@ -1140,7 +1143,12 @@ mod tests {
         // Merge commit and other commit
         assert_eq!(
             index.heads(&mut [id_5.clone(), id_3.clone()].iter()),
-            vec![id_3, id_5]
+            vec![id_3.clone(), id_5.clone()]
+        );
+
+        assert_eq!(
+            index.all_heads_for_gc().unwrap().collect_vec(),
+            vec![id_3.clone(), id_5.clone()]
         );
     }
 }
