@@ -14,7 +14,7 @@
 
 #![allow(missing_docs)]
 
-use std::fs::File;
+use std::fs::{self, File};
 use std::path::{Component, Path, PathBuf};
 use std::{io, iter};
 
@@ -42,6 +42,19 @@ impl<T> IoResultExt<T> for io::Result<T> {
     }
 }
 
+/// Creates a directory or does nothing if the directory already exists.
+///
+/// Returns the underlying error if the directory can't be created.
+/// The function will also fail if intermediate directories on the path do not
+/// already exist.
+pub fn create_or_reuse_dir(dirname: &Path) -> io::Result<()> {
+    match fs::create_dir(dirname) {
+        Ok(()) => Ok(()),
+        Err(_) if dirname.is_dir() => Ok(()),
+        Err(e) => Err(e),
+    }
+}
+
 /// Turns the given `to` path into relative path starting from the `from` path.
 ///
 /// Both `from` and `to` paths are supposed to be absolute and normalized in the
@@ -104,6 +117,7 @@ pub fn persist_content_addressed_temp_file<P: AsRef<Path>>(
             Ok(file) => Ok(file),
             Err(PersistError { error, file: _ }) => {
                 if let Ok(existing_file) = File::open(new_path) {
+                    // TODO: Update mtime to help GC keep this file
                     Ok(existing_file)
                 } else {
                     Err(error)
