@@ -26,9 +26,10 @@ use itertools::Itertools;
 use tempfile::NamedTempFile;
 use thiserror::Error;
 
-use crate::backend::{BackendError, CommitId, ObjectId};
+use crate::backend::{BackendError, CommitId};
 use crate::commit::Commit;
 use crate::git_backend::GitBackend;
+use crate::object_id::ObjectId;
 use crate::op_store::{RefTarget, RefTargetOptionExt, RemoteRef, RemoteRefState};
 use crate::refs::BranchPushUpdate;
 use crate::repo::{MutableRepo, Repo};
@@ -168,13 +169,13 @@ fn resolve_git_ref_to_commit_id(
 
 #[derive(Error, Debug)]
 pub enum GitImportError {
-    #[error("Failed to read Git HEAD target commit {id}: {err}", id=id.hex())]
+    #[error("Failed to read Git HEAD target commit {id}", id=id.hex())]
     MissingHeadTarget {
         id: CommitId,
         #[source]
         err: BackendError,
     },
-    #[error("Ancestor of Git ref {ref_name} is missing: {err}")]
+    #[error("Ancestor of Git ref {ref_name} is missing")]
     MissingRefAncestor {
         ref_name: String,
         #[source]
@@ -185,9 +186,9 @@ pub enum GitImportError {
         name = REMOTE_NAME_FOR_LOCAL_GIT_REPO
     )]
     RemoteReservedForLocalGitRepo,
-    #[error("Unexpected backend error when importing refs: {0}")]
+    #[error("Unexpected backend error when importing refs")]
     InternalBackend(#[source] BackendError),
-    #[error("Unexpected git error when importing refs: {0}")]
+    #[error("Unexpected git error when importing refs")]
     InternalGitError(#[source] Box<dyn std::error::Error + Send + Sync>),
     #[error("The repo is not backed by a Git repo")]
     UnexpectedBackend,
@@ -239,55 +240,33 @@ pub fn import_some_refs(
     let git_backend = get_git_backend(store).ok_or(GitImportError::UnexpectedBackend)?;
     let git_repo = git_backend.git_repo();
 
-    // TODO: Should this be a separate function? We may not always want to import
-    // the Git HEAD (and add it to our set of heads).
-    let old_git_head = mut_repo.view().git_head();
-    let changed_git_head = if let Ok(head_git_commit) = git_repo.head_commit() {
-        // The current HEAD is not added to `hidable_git_heads` because HEAD move
-        // doesn't automatically mean the old HEAD branch has been rewritten.
-        let head_commit_id = CommitId::from_bytes(head_git_commit.id().as_bytes());
-        let new_head_target = RefTarget::normal(head_commit_id);
-        (*old_git_head != new_head_target).then_some(new_head_target)
-    } else {
-        old_git_head.is_present().then(RefTarget::absent)
-    };
-
     let RefsToImport {
         changed_git_refs,
         changed_remote_refs,
     } = diff_refs_to_import(mut_repo.view(), &git_repo, git_ref_filter)?;
 
-    // Bulk-import all reachable Git commits to the backend to reduce overhead of
-    // table merging.
-    let head_ids = itertools::chain(
-        &changed_git_head,
-        changed_git_refs.iter().map(|(_, new_target)| new_target),
-        // changed_remote_refs might contain new_targets that are not in changed_git_refs,
-        // but such targets should have already been imported to the backend.
-    )
-    .flat_map(|target| target.added_ids());
-    let heads_imported = git_backend
-        .import_head_commits(head_ids, store.use_tree_conflict_format())
-        .is_ok();
+    // Bulk-import all reachable Git commits to the backend to reduce overhead
+    // of table merging and ref updates.
+    //
+    // changed_remote_refs might contain new_targets that are not in
+    // changed_git_refs, but such targets should have already been imported to
+    // the backend.
+    let index = mut_repo.index();
+    let missing_head_ids = changed_git_refs
+        .iter()
+        .flat_map(|(_, new_target)| new_target.added_ids())
+        .filter(|&id| !index.has_id(id));
+    let heads_imported = git_backend.import_head_commits(missing_head_ids).is_ok();
 
     // Import new remote heads
     let mut head_commits = Vec::new();
     let get_commit = |id| {
         // If bulk-import failed, try again to find bad head or ref.
-        if !heads_imported {
-            git_backend.import_head_commits([id], store.use_tree_conflict_format())?;
+        if !heads_imported && !index.has_id(id) {
+            git_backend.import_head_commits([id])?;
         }
         store.get_commit(id)
     };
-    if let Some(new_head_target) = &changed_git_head {
-        for id in new_head_target.added_ids() {
-            let commit = get_commit(id).map_err(|err| GitImportError::MissingHeadTarget {
-                id: id.clone(),
-                err,
-            })?;
-            head_commits.push(commit);
-        }
-    }
     for (ref_name, (_, new_target)) in &changed_remote_refs {
         for id in new_target.added_ids() {
             let commit = get_commit(id).map_err(|err| GitImportError::MissingRefAncestor {
@@ -304,9 +283,6 @@ pub fn import_some_refs(
         .map_err(GitImportError::InternalBackend)?;
 
     // Apply the change that happened in git since last time we imported refs.
-    if let Some(new_head_target) = changed_git_head {
-        mut_repo.set_git_head_target(new_head_target);
-    }
     for (full_name, new_target) in changed_git_refs {
         mut_repo.set_git_ref_target(&full_name, new_target);
     }
@@ -527,9 +503,54 @@ fn pinned_commit_ids(view: &View) -> impl Iterator<Item = &CommitId> {
     .flat_map(|target| target.added_ids())
 }
 
+/// Imports `HEAD@git` from the underlying Git repo.
+///
+/// Unlike `import_refs()`, the old HEAD branch is not abandoned because HEAD
+/// move doesn't always mean the old HEAD branch has been rewritten.
+///
+/// Unlike `reset_head()`, this function doesn't move the working-copy commit to
+/// the child of the new `HEAD@git` revision.
+pub fn import_head(mut_repo: &mut MutableRepo) -> Result<(), GitImportError> {
+    let store = mut_repo.store();
+    let git_backend = get_git_backend(store).ok_or(GitImportError::UnexpectedBackend)?;
+    let git_repo = git_backend.git_repo();
+
+    let old_git_head = mut_repo.view().git_head();
+    let new_git_head_id = if let Ok(oid) = git_repo.head_id() {
+        Some(CommitId::from_bytes(oid.as_bytes()))
+    } else {
+        None
+    };
+    if old_git_head.as_resolved() == Some(&new_git_head_id) {
+        return Ok(());
+    }
+
+    // Import new head
+    if let Some(head_id) = &new_git_head_id {
+        let index = mut_repo.index();
+        if !index.has_id(head_id) {
+            git_backend.import_head_commits([head_id]).map_err(|err| {
+                GitImportError::MissingHeadTarget {
+                    id: head_id.clone(),
+                    err,
+                }
+            })?;
+        }
+        // It's unlikely the imported commits were missing, but I/O-related
+        // error can still occur.
+        store
+            .get_commit(head_id)
+            .and_then(|commit| mut_repo.add_head(&commit))
+            .map_err(GitImportError::InternalBackend)?;
+    }
+
+    mut_repo.set_git_head_target(RefTarget::resolved(new_git_head_id));
+    Ok(())
+}
+
 #[derive(Error, Debug)]
 pub enum GitExportError {
-    #[error("Git error: {0}")]
+    #[error("Git error")]
     InternalGitError(#[source] Box<dyn std::error::Error + Send + Sync>),
     #[error("The repo is not backed by a Git repo")]
     UnexpectedBackend,
@@ -571,10 +592,10 @@ pub enum FailedRefExportReason {
     #[error("Modified ref had been deleted in Git")]
     ModifiedInJjDeletedInGit,
     /// Failed to delete the ref from the Git repo
-    #[error("Failed to delete: {0}")]
+    #[error("Failed to delete")]
     FailedToDelete(#[source] Box<gix::reference::edit::Error>),
     /// Failed to set the ref in the Git repo
-    #[error("Failed to set: {0}")]
+    #[error("Failed to set")]
     FailedToSet(#[source] Box<gix::reference::edit::Error>),
 }
 
@@ -1060,10 +1081,10 @@ pub enum GitFetchError {
         chars = INVALID_REFSPEC_CHARS.iter().join("`, `")
     )]
     InvalidBranchPattern,
-    #[error("Failed to import Git refs: {0}")]
+    #[error("Failed to import Git refs")]
     GitImportError(#[from] GitImportError),
     // TODO: I'm sure there are other errors possible, such as transport-level errors.
-    #[error("Unexpected git error when fetching: {0}")]
+    #[error("Unexpected git error when fetching")]
     InternalGitError(#[from] git2::Error),
 }
 
@@ -1175,7 +1196,7 @@ pub enum GitPushError {
     RefUpdateRejected(Vec<String>),
     // TODO: I'm sure there are other errors possible, such as transport-level errors,
     // and errors caused by the remote rejecting the push.
-    #[error("Unexpected git error when pushing: {0}")]
+    #[error("Unexpected git error when pushing")]
     InternalGitError(#[from] git2::Error),
 }
 
@@ -1459,9 +1480,9 @@ pub struct SubmoduleConfig {
 
 #[derive(Error, Debug)]
 pub enum GitConfigParseError {
-    #[error("Unexpected io error when parsing config: {0}")]
+    #[error("Unexpected io error when parsing config")]
     IoError(#[from] std::io::Error),
-    #[error("Unexpected git error when parsing config: {0}")]
+    #[error("Unexpected git error when parsing config")]
     InternalGitError(#[from] git2::Error),
 }
 
