@@ -15,11 +15,13 @@
 #![allow(missing_docs)]
 
 use std::any::Any;
+use std::collections::HashSet;
 use std::fmt::{Debug, Error, Formatter};
 use std::io::{Cursor, Read};
 use std::path::{Path, PathBuf};
-use std::process::ExitStatus;
+use std::process::{Command, ExitStatus};
 use std::sync::{Arc, Mutex, MutexGuard};
+use std::time::SystemTime;
 use std::{fs, io, str};
 
 use async_trait::async_trait;
@@ -32,12 +34,14 @@ use thiserror::Error;
 use crate::backend::{
     make_root_commit, Backend, BackendError, BackendInitError, BackendLoadError, BackendResult,
     ChangeId, Commit, CommitId, Conflict, ConflictId, ConflictTerm, FileId, MergedTreeId,
-    MillisSinceEpoch, ObjectId, SecureSig, Signature, SigningFn, SymlinkId, Timestamp, Tree,
-    TreeId, TreeValue,
+    MillisSinceEpoch, SecureSig, Signature, SigningFn, SymlinkId, Timestamp, Tree, TreeId,
+    TreeValue,
 };
 use crate::file_util::{IoResultExt as _, PathError};
+use crate::index::Index;
 use crate::lock::FileLock;
 use crate::merge::{Merge, MergeBuilder};
+use crate::object_id::ObjectId;
 use crate::repo_path::{RepoPath, RepoPathComponentBuf};
 use crate::settings::UserSettings;
 use crate::stacked_table::{
@@ -52,9 +56,9 @@ const CONFLICT_SUFFIX: &str = ".jjconflict";
 
 #[derive(Debug, Error)]
 pub enum GitBackendInitError {
-    #[error("Failed to initialize git repository: {0}")]
+    #[error("Failed to initialize git repository")]
     InitRepository(#[source] gix::init::Error),
-    #[error("Failed to open git repository: {0}")]
+    #[error("Failed to open git repository")]
     OpenRepository(#[source] gix::open::Error),
     #[error(transparent)]
     Path(PathError),
@@ -68,7 +72,7 @@ impl From<Box<GitBackendInitError>> for BackendInitError {
 
 #[derive(Debug, Error)]
 pub enum GitBackendLoadError {
-    #[error("Failed to open git repository: {0}")]
+    #[error("Failed to open git repository")]
     OpenRepository(#[source] gix::open::Error),
     #[error(transparent)]
     Path(PathError),
@@ -83,9 +87,9 @@ impl From<Box<GitBackendLoadError>> for BackendLoadError {
 /// `GitBackend`-specific error that may occur after the backend is loaded.
 #[derive(Debug, Error)]
 pub enum GitBackendError {
-    #[error("Failed to read non-git metadata: {0}")]
+    #[error("Failed to read non-git metadata")]
     ReadMetadata(#[source] TableStoreError),
-    #[error("Failed to write non-git metadata: {0}")]
+    #[error("Failed to write non-git metadata")]
     WriteMetadata(#[source] TableStoreError),
 }
 
@@ -97,7 +101,7 @@ impl From<GitBackendError> for BackendError {
 
 #[derive(Debug, Error)]
 pub enum GitGcError {
-    #[error("Failed to run git gc command: {0}")]
+    #[error("Failed to run git gc command")]
     GcCommand(#[source] std::io::Error),
     #[error("git gc command exited with an error: {0}")]
     GcCommandErrorStatus(ExitStatus),
@@ -115,6 +119,8 @@ pub struct GitBackend {
     empty_tree_id: TreeId,
     extra_metadata_store: TableStore,
     cached_extra_metadata: Mutex<Option<Arc<ReadonlyTable>>>,
+    /// Whether tree of imported commit should be promoted to non-legacy format.
+    imported_commit_uses_tree_conflict_format: bool,
 }
 
 impl GitBackend {
@@ -122,7 +128,11 @@ impl GitBackend {
         "git"
     }
 
-    fn new(base_repo: gix::ThreadSafeRepository, extra_metadata_store: TableStore) -> Self {
+    fn new(
+        base_repo: gix::ThreadSafeRepository,
+        extra_metadata_store: TableStore,
+        imported_commit_uses_tree_conflict_format: bool,
+    ) -> Self {
         let repo = Mutex::new(base_repo.to_thread_local());
         let root_commit_id = CommitId::from_bytes(&[0; HASH_LENGTH]);
         let root_change_id = ChangeId::from_bytes(&[0; CHANGE_ID_LENGTH]);
@@ -135,6 +145,7 @@ impl GitBackend {
             empty_tree_id,
             extra_metadata_store,
             cached_extra_metadata: Mutex::new(None),
+            imported_commit_uses_tree_conflict_format,
         }
     }
 
@@ -150,7 +161,7 @@ impl GitBackend {
             gix_open_opts_from_settings(settings),
         )
         .map_err(GitBackendInitError::InitRepository)?;
-        Self::init_with_repo(store_path, git_repo_path, git_repo)
+        Self::init_with_repo(settings, store_path, git_repo_path, git_repo)
     }
 
     /// Initializes backend by creating a new Git repo at the specified
@@ -174,7 +185,7 @@ impl GitBackend {
         )
         .map_err(GitBackendInitError::InitRepository)?;
         let git_repo_path = workspace_root.join(".git");
-        Self::init_with_repo(store_path, &git_repo_path, git_repo)
+        Self::init_with_repo(settings, store_path, &git_repo_path, git_repo)
     }
 
     /// Initializes backend with an existing Git repo at the specified path.
@@ -194,10 +205,11 @@ impl GitBackend {
             gix_open_opts_from_settings(settings),
         )
         .map_err(GitBackendInitError::OpenRepository)?;
-        Self::init_with_repo(store_path, git_repo_path, git_repo)
+        Self::init_with_repo(settings, store_path, git_repo_path, git_repo)
     }
 
     fn init_with_repo(
+        settings: &UserSettings,
         store_path: &Path,
         git_repo_path: &Path,
         git_repo: gix::ThreadSafeRepository,
@@ -227,7 +239,11 @@ impl GitBackend {
                 .map_err(GitBackendInitError::Path)?;
         };
         let extra_metadata_store = TableStore::init(extra_path, HASH_LENGTH);
-        Ok(GitBackend::new(git_repo, extra_metadata_store))
+        Ok(GitBackend::new(
+            git_repo,
+            extra_metadata_store,
+            settings.use_tree_conflict_format(),
+        ))
     }
 
     pub fn load(
@@ -250,7 +266,11 @@ impl GitBackend {
         )
         .map_err(GitBackendLoadError::OpenRepository)?;
         let extra_metadata_store = TableStore::load(store_path.join("extra"), HASH_LENGTH);
-        Ok(GitBackend::new(repo, extra_metadata_store))
+        Ok(GitBackend::new(
+            repo,
+            extra_metadata_store,
+            settings.use_tree_conflict_format(),
+        ))
     }
 
     fn lock_git_repo(&self) -> MutexGuard<'_, gix::Repository> {
@@ -316,42 +336,44 @@ impl GitBackend {
     }
 
     /// Imports the given commits and ancestors from the backing Git repo.
+    ///
+    /// The `head_ids` may contain commits that have already been imported, but
+    /// the caller should filter them out to eliminate redundant I/O processing.
     #[tracing::instrument(skip(self, head_ids))]
     pub fn import_head_commits<'a>(
         &self,
         head_ids: impl IntoIterator<Item = &'a CommitId>,
-        uses_tree_conflict_format: bool,
     ) -> BackendResult<()> {
-        let table = self.cached_extra_metadata_table()?;
-        let mut missing_head_ids = head_ids
+        let head_ids: HashSet<&CommitId> = head_ids
             .into_iter()
-            .filter(|&id| *id != self.root_commit_id && table.get_value(id.as_bytes()).is_none())
-            .collect_vec();
-        if missing_head_ids.is_empty() {
+            .filter(|&id| *id != self.root_commit_id)
+            .collect();
+        if head_ids.is_empty() {
             return Ok(());
         }
 
+        // Create no-gc ref even if known to the extras table. Concurrent GC
+        // process might have deleted the no-gc ref.
+        let locked_repo = self.lock_git_repo();
+        locked_repo
+            .edit_references(head_ids.iter().copied().map(to_no_gc_ref_update))
+            .map_err(|err| BackendError::Other(Box::new(err)))?;
+
         // These commits are imported from Git. Make our change ids persist (otherwise
         // future write_commit() could reassign new change id.)
         tracing::debug!(
-            heads_count = missing_head_ids.len(),
+            heads_count = head_ids.len(),
             "import extra metadata entries"
         );
-        let locked_repo = self.lock_git_repo();
         let (table, table_lock) = self.read_extra_metadata_table_locked()?;
         let mut mut_table = table.start_mutation();
-        // Concurrent write_commit() might have updated the table before taking a lock.
-        missing_head_ids.retain(|&id| mut_table.get_value(id.as_bytes()).is_none());
         import_extra_metadata_entries_from_heads(
             &locked_repo,
             &mut mut_table,
             &table_lock,
-            &missing_head_ids,
-            uses_tree_conflict_format,
+            &head_ids,
+            self.imported_commit_uses_tree_conflict_format,
         )?;
-        for &id in &missing_head_ids {
-            prevent_gc(&locked_repo, id)?;
-        }
         self.save_extra_metadata_table(mut_table, &table_lock)
     }
 
@@ -565,16 +587,120 @@ fn deserialize_extras(commit: &mut Commit, bytes: &[u8]) {
     }
 }
 
-/// Creates a ref in refs/jj/. Used for preventing GC of commits we create.
-fn prevent_gc(git_repo: &gix::Repository, id: &CommitId) -> Result<(), BackendError> {
+/// Returns `RefEdit` that will create a ref in `refs/jj/keep` if not exist.
+/// Used for preventing GC of commits we create.
+fn to_no_gc_ref_update(id: &CommitId) -> gix::refs::transaction::RefEdit {
+    let name = format!("{NO_GC_REF_NAMESPACE}{}", id.hex());
+    let new = gix::refs::Target::Peeled(validate_git_object_id(id).unwrap());
+    let expected = gix::refs::transaction::PreviousValue::ExistingMustMatch(new.clone());
+    gix::refs::transaction::RefEdit {
+        change: gix::refs::transaction::Change::Update {
+            log: gix::refs::transaction::LogChange {
+                message: "used by jj".into(),
+                ..Default::default()
+            },
+            expected,
+            new,
+        },
+        name: name.try_into().unwrap(),
+        deref: false,
+    }
+}
+
+fn to_ref_deletion(git_ref: gix::refs::Reference) -> gix::refs::transaction::RefEdit {
+    let expected = gix::refs::transaction::PreviousValue::ExistingMustMatch(git_ref.target);
+    gix::refs::transaction::RefEdit {
+        change: gix::refs::transaction::Change::Delete {
+            expected,
+            log: gix::refs::transaction::RefLog::AndReference,
+        },
+        name: git_ref.name,
+        deref: false,
+    }
+}
+
+/// Recreates `refs/jj/keep` refs for the `new_heads`, and removes the other
+/// unreachable and non-head refs.
+fn recreate_no_gc_refs(
+    git_repo: &gix::Repository,
+    new_heads: impl IntoIterator<Item = CommitId>,
+    keep_newer: SystemTime,
+) -> Result<(), BackendError> {
+    // Calculate diff between existing no-gc refs and new heads.
+    let new_heads: HashSet<CommitId> = new_heads.into_iter().collect();
+    let mut no_gc_refs_to_keep_count: usize = 0;
+    let mut no_gc_refs_to_delete: Vec<gix::refs::Reference> = Vec::new();
+    let git_references = git_repo
+        .references()
+        .map_err(|err| BackendError::Other(err.into()))?;
+    let no_gc_refs_iter = git_references
+        .prefixed(NO_GC_REF_NAMESPACE)
+        .map_err(|err| BackendError::Other(err.into()))?;
+    for git_ref in no_gc_refs_iter {
+        let git_ref = git_ref.map_err(BackendError::Other)?.detach();
+        let oid = git_ref.target.try_id().ok_or_else(|| {
+            let name = git_ref.name.as_bstr();
+            BackendError::Other(format!("Symbolic no-gc ref found: {name}").into())
+        })?;
+        let id = CommitId::from_bytes(oid.as_bytes());
+        let name_good = git_ref.name.as_bstr()[NO_GC_REF_NAMESPACE.len()..] == id.hex();
+        if new_heads.contains(&id) && name_good {
+            no_gc_refs_to_keep_count += 1;
+            continue;
+        }
+        // Check timestamp of loose ref, but this is still racy on re-import
+        // because:
+        // - existing packed ref won't be demoted to loose ref
+        // - existing loose ref won't be touched
+        //
+        // TODO: might be better to switch to a dummy merge, where new no-gc ref
+        // will always have a unique name. Doing that with the current
+        // ref-per-head strategy would increase the number of the no-gc refs.
+        // https://github.com/martinvonz/jj/pull/2659#issuecomment-1837057782
+        let loose_ref_path = git_repo.path().join(git_ref.name.to_path());
+        if let Ok(metadata) = loose_ref_path.metadata() {
+            let mtime = metadata.modified().expect("unsupported platform?");
+            if mtime > keep_newer {
+                tracing::trace!(?git_ref, "not deleting new");
+                no_gc_refs_to_keep_count += 1;
+                continue;
+            }
+        }
+        // Also deletes no-gc ref of random name created by old jj.
+        tracing::trace!(?git_ref, ?name_good, "will delete");
+        no_gc_refs_to_delete.push(git_ref);
+    }
+    tracing::info!(
+        new_heads_count = new_heads.len(),
+        no_gc_refs_to_keep_count,
+        no_gc_refs_to_delete_count = no_gc_refs_to_delete.len(),
+        "collected reachable refs"
+    );
+
+    // It's slow to delete packed refs one by one, so update refs all at once.
+    let ref_edits = itertools::chain(
+        no_gc_refs_to_delete.into_iter().map(to_ref_deletion),
+        new_heads.iter().map(to_no_gc_ref_update),
+    );
     git_repo
-        .reference(
-            format!("{NO_GC_REF_NAMESPACE}{}", id.hex()),
-            validate_git_object_id(id).unwrap(),
-            gix::refs::transaction::PreviousValue::Any,
-            "used by jj",
-        )
-        .map_err(|err| BackendError::Other(Box::new(err)))?;
+        .edit_references(ref_edits)
+        .map_err(|err| BackendError::Other(err.into()))?;
+
+    Ok(())
+}
+
+fn run_git_gc(git_dir: &Path) -> Result<(), GitGcError> {
+    let mut git = Command::new("git");
+    git.arg("--git-dir=."); // turn off discovery
+    git.arg("gc");
+    // Don't specify it by GIT_DIR/--git-dir. On Windows, the "\\?\" path might
+    // not be supported by git.
+    git.current_dir(git_dir);
+    // TODO: pass output to UI layer instead of printing directly here
+    let status = git.status().map_err(GitGcError::GcCommand)?;
+    if !status.success() {
+        return Err(GitGcError::GcCommandErrorStatus(status));
+    }
     Ok(())
 }
 
@@ -625,10 +751,14 @@ fn import_extra_metadata_entries_from_heads(
     git_repo: &gix::Repository,
     mut_table: &mut MutableTable,
     _table_lock: &FileLock,
-    missing_head_ids: &[&CommitId],
+    head_ids: &HashSet<&CommitId>,
     uses_tree_conflict_format: bool,
 ) -> BackendResult<()> {
-    let mut work_ids = missing_head_ids.iter().map(|&id| id.clone()).collect_vec();
+    let mut work_ids = head_ids
+        .iter()
+        .filter(|&id| mut_table.get_value(id.as_bytes()).is_none())
+        .map(|&id| id.clone())
+        .collect_vec();
     while let Some(id) = work_ids.pop() {
         let git_object = git_repo
             .find_object(validate_git_object_id(&id)?)
@@ -918,8 +1048,7 @@ impl Backend for GitBackend {
             // imported by jj < 0.8.0 might not have extras (#924).
             // https://github.com/martinvonz/jj/issues/2343
             tracing::info!("unimported Git commit found");
-            let uses_tree_conflict_format = false;
-            self.import_head_commits([id], uses_tree_conflict_format)?;
+            self.import_head_commits([id])?;
             let table = self.cached_extra_metadata_table()?;
             let extras = table.get_value(id.as_bytes()).unwrap();
             deserialize_extras(&mut commit, extras);
@@ -1023,7 +1152,9 @@ impl Backend for GitBackend {
 
         // Everything up to this point had no permanent effect on the repo except
         // GC-able objects
-        prevent_gc(&locked_repo, &id)?;
+        locked_repo
+            .edit_reference(to_no_gc_ref_update(&id))
+            .map_err(|err| BackendError::Other(Box::new(err)))?;
 
         // Update the signature to match the one that was actually written to the object
         // store
@@ -1034,16 +1165,20 @@ impl Backend for GitBackend {
         Ok((id, contents))
     }
 
-    fn gc(&self) -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
-        let mut git = std::process::Command::new("git");
-        git.env("GIT_DIR", self.git_repo_path());
-        git.args(["gc"]);
-        // TODO: pass output to UI layer instead of printing directly here
-        let status = git.status().map_err(GitGcError::GcCommand)?;
-        if !status.success() {
-            return Err(Box::new(GitGcError::GcCommandErrorStatus(status)));
-        }
-        Ok(())
+    #[tracing::instrument(skip(self, index))]
+    fn gc(&self, index: &dyn Index, keep_newer: SystemTime) -> BackendResult<()> {
+        let git_repo = self.lock_git_repo();
+        let new_heads = index
+            .all_heads_for_gc()
+            .map_err(|err| BackendError::Other(err.into()))?
+            .filter(|id| *id != self.root_commit_id);
+        recreate_no_gc_refs(&git_repo, new_heads, keep_newer)?;
+        // TODO: remove unreachable entries from extras table if segment file
+        // mtime <= keep_newer? (it won't be consistent with no-gc refs
+        // preserved by the keep_newer timestamp though)
+        // TODO: remove unreachable extras table segments
+        // TODO: pass in keep_newer to "git gc" command
+        run_git_gc(self.git_repo_path()).map_err(|err| BackendError::Other(err.into()))
     }
 }
 
@@ -1166,7 +1301,14 @@ mod tests {
     #[test_case(false; "legacy tree format")]
     #[test_case(true; "tree-level conflict format")]
     fn read_plain_git_commit(uses_tree_conflict_format: bool) {
-        let settings = user_settings();
+        let settings = {
+            let config = config::Config::builder()
+                .set_override("format.tree-level-conflicts", uses_tree_conflict_format)
+                .unwrap()
+                .build()
+                .unwrap();
+            UserSettings::from_config(config)
+        };
         let temp_dir = testutils::new_temp_dir();
         let store_path = temp_dir.path();
         let git_repo_path = temp_dir.path().join("git");
@@ -1229,9 +1371,7 @@ mod tests {
         let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
 
         // Import the head commit and its ancestors
-        backend
-            .import_head_commits([&commit_id2], uses_tree_conflict_format)
-            .unwrap();
+        backend.import_head_commits([&commit_id2]).unwrap();
         // Ref should be created only for the head commit
         let git_refs = backend
             .open_git_repo()
@@ -1619,6 +1759,7 @@ mod tests {
         let settings = user_settings();
         let temp_dir = testutils::new_temp_dir();
         let backend = GitBackend::init_internal(&settings, temp_dir.path()).unwrap();
+        let git_repo = backend.open_git_repo().unwrap();
         let signature = Signature {
             name: "Someone".to_string(),
             email: "someone@example.com".to_string(),
@@ -1638,14 +1779,65 @@ mod tests {
             secure_sig: None,
         };
         let commit_id = backend.write_commit(commit, None).unwrap().0;
-        let git_refs = backend
-            .open_git_repo()
-            .unwrap()
+        let git_refs: Vec<_> = git_repo
             .references_glob("refs/jj/keep/*")
             .unwrap()
-            .map(|git_ref| git_ref.unwrap().target().unwrap())
-            .collect_vec();
-        assert_eq!(git_refs, vec![git_id(&commit_id)]);
+            .try_collect()
+            .unwrap();
+        assert!(git_refs
+            .iter()
+            .any(|git_ref| git_ref.target().unwrap() == git_id(&commit_id)));
+
+        // Concurrently-running GC deletes the ref, leaving the extra metadata.
+        for mut git_ref in git_refs {
+            git_ref.delete().unwrap();
+        }
+        // Re-imported commit should have new ref.
+        backend.import_head_commits([&commit_id]).unwrap();
+        let git_refs: Vec<_> = git_repo
+            .references_glob("refs/jj/keep/*")
+            .unwrap()
+            .try_collect()
+            .unwrap();
+        assert!(git_refs
+            .iter()
+            .any(|git_ref| git_ref.target().unwrap() == git_id(&commit_id)));
+    }
+
+    #[test]
+    fn import_head_commits_duplicates() {
+        let settings = user_settings();
+        let temp_dir = testutils::new_temp_dir();
+        let backend = GitBackend::init_internal(&settings, temp_dir.path()).unwrap();
+        let git_repo = backend.open_git_repo().unwrap();
+
+        let signature = git2::Signature::now("Someone", "someone@example.com").unwrap();
+        let empty_tree_id = Oid::from_str("4b825dc642cb6eb9a060e54bf8d69288fbee4904").unwrap();
+        let empty_tree = git_repo.find_tree(empty_tree_id).unwrap();
+        let git_commit_id = git_repo
+            .commit(
+                Some("refs/heads/main"),
+                &signature,
+                &signature,
+                "git commit message",
+                &empty_tree,
+                &[],
+            )
+            .unwrap();
+        let commit_id = CommitId::from_bytes(git_commit_id.as_bytes());
+
+        // Ref creation shouldn't fail because of duplicated head ids.
+        backend
+            .import_head_commits([&commit_id, &commit_id])
+            .unwrap();
+        let git_refs: Vec<_> = git_repo
+            .references_glob("refs/jj/keep/*")
+            .unwrap()
+            .try_collect()
+            .unwrap();
+        assert!(git_refs
+            .iter()
+            .any(|git_ref| git_ref.target().unwrap() == git_commit_id));
     }
 
     #[test]
