@@ -21,8 +21,9 @@ use std::rc::Rc;
 use itertools::Itertools as _;
 use once_cell::unsync::OnceCell;
 
-use crate::backend::{self, ChangeId, CommitId, ObjectId};
-use crate::index::{HexPrefix, PrefixResolution};
+use crate::backend::{ChangeId, CommitId};
+use crate::hex_util;
+use crate::object_id::{HexPrefix, ObjectId, PrefixResolution};
 use crate::repo::Repo;
 use crate::revset::{DefaultSymbolResolver, RevsetExpression};
 
@@ -421,7 +422,7 @@ where
 
         // Left/right neighbors should have unique short keys. For the current chunk,
         // we need to look up full-length keys.
-        let unique_len = |a: &[u8], b: &[u8]| backend::common_hex_len(a, b) + 1;
+        let unique_len = |a: &[u8], b: &[u8]| hex_util::common_hex_len(a, b) + 1;
         let neighbor_lens = left
             .iter()
             .chain(&right)
@@ -444,7 +445,7 @@ fn unwrap_as_short_key<const N: usize>(key_bytes: &[u8]) -> &[u8; N] {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::backend::{ChangeId, ObjectId};
+    use crate::backend::ChangeId;
 
     #[derive(Clone, Copy, Eq, PartialEq)]
     struct Position(usize);
