@@ -42,7 +42,7 @@ use thiserror::Error;
 use tracing::{instrument, trace_span};
 
 use crate::backend::{
-    BackendError, FileId, MergedTreeId, MillisSinceEpoch, ObjectId, SymlinkId, TreeId, TreeValue,
+    BackendError, FileId, MergedTreeId, MillisSinceEpoch, SymlinkId, TreeId, TreeValue,
 };
 use crate::commit::Commit;
 use crate::conflicts::{self, materialize_tree_value, MaterializedTreeValue};
@@ -56,6 +56,7 @@ use crate::matchers::{
 };
 use crate::merge::{Merge, MergeBuilder, MergedTreeValue};
 use crate::merged_tree::{MergedTree, MergedTreeBuilder};
+use crate::object_id::ObjectId;
 use crate::op_store::{OperationId, WorkspaceId};
 use crate::repo_path::{RepoPath, RepoPathBuf, RepoPathComponent};
 use crate::settings::HumanByteSize;
@@ -63,7 +64,7 @@ use crate::store::Store;
 use crate::tree::Tree;
 use crate::working_copy::{
     CheckoutError, CheckoutStats, LockedWorkingCopy, ResetError, SnapshotError, SnapshotOptions,
-    SnapshotProgress, WorkingCopy, WorkingCopyStateError,
+    SnapshotProgress, WorkingCopy, WorkingCopyFactory, WorkingCopyStateError,
 };
 
 #[cfg(unix)]
@@ -476,28 +477,28 @@ struct DirectoryToVisit<'a> {
 
 #[derive(Debug, Error)]
 pub enum TreeStateError {
-    #[error("Reading tree state from {path}: {source}")]
+    #[error("Reading tree state from {path}")]
     ReadTreeState {
         path: PathBuf,
         source: std::io::Error,
     },
-    #[error("Decoding tree state from {path}: {source}")]
+    #[error("Decoding tree state from {path}")]
     DecodeTreeState {
         path: PathBuf,
         source: prost::DecodeError,
     },
-    #[error("Writing tree state to temporary file {path}: {source}")]
+    #[error("Writing tree state to temporary file {path}")]
     WriteTreeState {
         path: PathBuf,
         source: std::io::Error,
     },
-    #[error("Persisting tree state to file {path}: {source}")]
+    #[error("Persisting tree state to file {path}")]
     PersistTreeState {
         path: PathBuf,
         source: std::io::Error,
     },
-    #[error("Filesystem monitor error: {0}")]
-    Fsmonitor(Box<dyn Error + Send + Sync>),
+    #[error("Filesystem monitor error")]
+    Fsmonitor(#[source] Box<dyn Error + Send + Sync>),
 }
 
 impl TreeState {
@@ -893,7 +894,7 @@ impl TreeState {
                                 continue;
                             }
                             let disk_path = tracked_path.to_fs_path(&self.working_copy_path);
-                            let metadata = match disk_path.metadata() {
+                            let metadata = match disk_path.symlink_metadata() {
                                 Ok(metadata) => metadata,
                                 Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
                                     continue;
@@ -1532,7 +1533,6 @@ impl LocalWorkingCopy {
         let proto = crate::protos::working_copy::Checkout {
             operation_id: operation_id.to_bytes(),
             workspace_id: workspace_id.as_str().to_string(),
-            ..Default::default()
         };
         let mut file = OpenOptions::new()
             .create_new(true)
@@ -1637,7 +1637,6 @@ impl LocalWorkingCopy {
         self.write_proto(crate::protos::working_copy::Checkout {
             operation_id: self.operation_id().to_bytes(),
             workspace_id: self.workspace_id().as_str().to_string(),
-            ..Default::default()
         });
     }
 
@@ -1654,6 +1653,36 @@ impl LocalWorkingCopy {
     }
 }
 
+pub struct LocalWorkingCopyFactory {}
+
+impl WorkingCopyFactory for LocalWorkingCopyFactory {
+    fn init_working_copy(
+        &self,
+        store: Arc<Store>,
+        working_copy_path: PathBuf,
+        state_path: PathBuf,
+        operation_id: OperationId,
+        workspace_id: WorkspaceId,
+    ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError> {
+        Ok(Box::new(LocalWorkingCopy::init(
+            store,
+            working_copy_path,
+            state_path,
+            operation_id,
+            workspace_id,
+        )?))
+    }
+
+    fn load_working_copy(
+        &self,
+        store: Arc<Store>,
+        working_copy_path: PathBuf,
+        state_path: PathBuf,
+    ) -> Box<dyn WorkingCopy> {
+        Box::new(LocalWorkingCopy::load(store, working_copy_path, state_path))
+    }
+}
+
 /// A working copy that's locked on disk. The lock is held until you call
 /// `finish()` or `discard()`.
 pub struct LockedLocalWorkingCopy {
@@ -1710,14 +1739,15 @@ impl LockedWorkingCopy for LockedLocalWorkingCopy {
         Ok(stats)
     }
 
-    fn reset(&mut self, new_tree: &MergedTree) -> Result<(), ResetError> {
+    fn reset(&mut self, commit: &Commit) -> Result<(), ResetError> {
+        let new_tree = commit.tree()?;
         self.wc
             .tree_state_mut()
             .map_err(|err| ResetError::Other {
                 message: "Failed to read the working copy state".to_string(),
                 err: err.into(),
             })?
-            .reset(new_tree)
+            .reset(&new_tree)
             .block_on()?;
         self.tree_state_dirty = true;
         Ok(())
