@@ -14,17 +14,20 @@
 
 #![allow(missing_docs)]
 
+use std::any::Any;
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::fmt::{Debug, Error, Formatter};
 use std::iter;
+use std::time::SystemTime;
 
 use itertools::Itertools as _;
 use once_cell::sync::Lazy;
 use thiserror::Error;
 
-use crate::backend::{id_type, CommitId, ObjectId, Timestamp};
+use crate::backend::{CommitId, MillisSinceEpoch, Timestamp};
 use crate::content_hash::ContentHash;
 use crate::merge::Merge;
+use crate::object_id::{id_type, HexPrefix, ObjectId, PrefixResolution};
 
 content_hash! {
     #[derive(PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
@@ -105,10 +108,14 @@ impl RefTarget {
         RefTarget { merge }
     }
 
+    /// Returns the underlying value if this target is non-conflicting.
+    pub fn as_resolved(&self) -> Option<&Option<CommitId>> {
+        self.merge.as_resolved()
+    }
+
     /// Returns id if this target is non-conflicting and points to a commit.
     pub fn as_normal(&self) -> Option<&CommitId> {
-        let maybe_id = self.merge.as_resolved()?;
-        maybe_id.as_ref()
+        self.as_resolved()?.as_ref()
     }
 
     /// Returns true if this target points to no commit.
@@ -266,8 +273,6 @@ content_hash! {
     pub struct View {
         /// All head commits
         pub head_ids: HashSet<CommitId>,
-        /// Heads of the set of public commits.
-        pub public_head_ids: HashSet<CommitId>,
         pub local_branches: BTreeMap<String, RefTarget>,
         pub tags: BTreeMap<String, RefTarget>,
         pub remote_views: BTreeMap<String, RemoteView>,
@@ -369,6 +374,28 @@ content_hash! {
     }
 }
 
+impl Operation {
+    pub fn make_root(empty_view_id: ViewId) -> Operation {
+        let timestamp = Timestamp {
+            timestamp: MillisSinceEpoch(0),
+            tz_offset: 0,
+        };
+        let metadata = OperationMetadata {
+            start_time: timestamp.clone(),
+            end_time: timestamp,
+            description: "".to_string(),
+            hostname: "".to_string(),
+            username: "".to_string(),
+            tags: HashMap::new(),
+        };
+        Operation {
+            view_id: empty_view_id,
+            parents: vec![],
+            metadata,
+        }
+    }
+}
+
 content_hash! {
     #[derive(PartialEq, Eq, Clone, Debug)]
     pub struct OperationMetadata {
@@ -384,19 +411,19 @@ content_hash! {
 
 #[derive(Debug, Error)]
 pub enum OpStoreError {
-    #[error("Object {hash} of type {object_type} not found: {source}")]
+    #[error("Object {hash} of type {object_type} not found")]
     ObjectNotFound {
         object_type: String,
         hash: String,
         source: Box<dyn std::error::Error + Send + Sync>,
     },
-    #[error("Error when reading object {hash} of type {object_type}: {source}")]
+    #[error("Error when reading object {hash} of type {object_type}")]
     ReadObject {
         object_type: String,
         hash: String,
         source: Box<dyn std::error::Error + Send + Sync>,
     },
-    #[error("Could not write object of type {object_type}: {source}")]
+    #[error("Could not write object of type {object_type}")]
     WriteObject {
         object_type: &'static str,
         source: Box<dyn std::error::Error + Send + Sync>,
@@ -408,8 +435,12 @@ pub enum OpStoreError {
 pub type OpStoreResult<T> = Result<T, OpStoreError>;
 
 pub trait OpStore: Send + Sync + Debug {
+    fn as_any(&self) -> &dyn Any;
+
     fn name(&self) -> &str;
 
+    fn root_operation_id(&self) -> &OperationId;
+
     fn read_view(&self, id: &ViewId) -> OpStoreResult<View>;
 
     fn write_view(&self, contents: &View) -> OpStoreResult<ViewId>;
@@ -417,6 +448,21 @@ pub trait OpStore: Send + Sync + Debug {
     fn read_operation(&self, id: &OperationId) -> OpStoreResult<Operation>;
 
     fn write_operation(&self, contents: &Operation) -> OpStoreResult<OperationId>;
+
+    /// Resolves an unambiguous operation ID prefix.
+    fn resolve_operation_id_prefix(
+        &self,
+        prefix: &HexPrefix,
+    ) -> OpStoreResult<PrefixResolution<OperationId>>;
+
+    /// Prunes unreachable operations and views.
+    ///
+    /// All operations and views reachable from the `head_ids` won't be
+    /// removed. In addition to that, objects created after `keep_newer` will be
+    /// preserved. This mitigates a risk of deleting new heads created
+    /// concurrently by another process.
+    // TODO: return stats?
+    fn gc(&self, head_ids: &[OperationId], keep_newer: SystemTime) -> OpStoreResult<()>;
 }
 
 #[cfg(test)]
