@@ -22,12 +22,13 @@ use itertools::Itertools;
 use pollster::FutureExt;
 use tracing::instrument;
 
-use crate::backend::{BackendError, BackendResult, CommitId, MergedTreeId, ObjectId};
+use crate::backend::{BackendError, BackendResult, CommitId, MergedTreeId};
 use crate::commit::Commit;
 use crate::dag_walk;
 use crate::index::Index;
 use crate::matchers::{Matcher, Visit};
 use crate::merged_tree::{MergedTree, MergedTreeBuilder};
+use crate::object_id::ObjectId;
 use crate::op_store::RefTarget;
 use crate::repo::{MutableRepo, Repo};
 use crate::repo_path::RepoPath;
@@ -158,12 +159,16 @@ pub fn rebase_commit_with_options(
             }
             EmptyBehaviour::AbandonAllEmpty => *parent.tree_id() == new_tree_id,
         };
-        if should_abandon {
-            mut_repo.record_abandoned_commit(old_commit.id().clone());
-            // Record old_commit as being succeeded by the parent for the purposes of
-            // the rebase.
+        // If the user runs `jj checkout foo`, then `jj rebase -s foo -d bar`, and we
+        // drop the checked out empty commit, then the user will unknowingly
+        // have done the equivalent of `jj edit foo` instead of `jj checkout
+        // foo`. Thus, we never allow dropping the working commit. See #2766 and
+        // #2760 for discussions.
+        if should_abandon && !mut_repo.view().is_wc_commit_id(old_commit.id()) {
+            // Record old_commit as being succeeded by the parent.
             // This ensures that when we stack commits, the second commit knows to
             // rebase on top of the parent commit, rather than the abandoned commit.
+            mut_repo.record_rewritten_commit(old_commit.id().clone(), parent.id().clone());
             return Ok(parent.clone());
         }
     }
@@ -451,10 +456,10 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         &mut self,
         old_commit_id: CommitId,
         new_commit_ids: Vec<CommitId>,
-        edit: bool,
+        abandoned_old_commit: bool,
     ) -> Result<(), BackendError> {
         // We arbitrarily pick a new working-copy commit among the candidates.
-        self.update_wc_commits(&old_commit_id, &new_commit_ids[0], edit)?;
+        self.update_wc_commits(&old_commit_id, &new_commit_ids[0], abandoned_old_commit)?;
 
         if let Some(branch_names) = self.branches.get(&old_commit_id).cloned() {
             let mut branch_updates = vec![];
@@ -491,7 +496,7 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         &mut self,
         old_commit_id: &CommitId,
         new_commit_id: &CommitId,
-        edit: bool,
+        abandoned_old_commit: bool,
     ) -> Result<(), BackendError> {
         let workspaces_to_update = self
             .mut_repo
@@ -502,7 +507,7 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         }
 
         let new_commit = self.mut_repo.store().get_commit(new_commit_id)?;
-        let new_wc_commit = if edit {
+        let new_wc_commit = if !abandoned_old_commit {
             new_commit
         } else {
             self.mut_repo
@@ -519,80 +524,73 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         Ok(())
     }
 
-    // TODO: Perhaps change the interface since it's not just about rebasing
-    // commits.
-    fn rebase_next(&mut self) -> Result<Option<RebasedDescendant>, TreeMergeError> {
-        while let Some(old_commit) = self.to_visit.pop() {
-            let old_commit_id = old_commit.id().clone();
-            if let Some(new_parent_ids) = self.parent_mapping.get(&old_commit_id).cloned() {
-                // This is a commit that had already been rebased before `self` was created
-                // (i.e. it's part of the input for this rebase). We don't need
-                // to rebase it, but we still want to update branches pointing
-                // to the old commit.
-                self.update_references(old_commit_id, new_parent_ids, true)?;
-                continue;
-            }
-            if let Some(divergent_ids) = self.divergent.get(&old_commit_id).cloned() {
-                // Leave divergent commits in place. Don't update `parent_mapping` since we
-                // don't want to rebase descendants either.
-                self.update_references(old_commit_id, divergent_ids, true)?;
-                continue;
-            }
-            let old_parent_ids = old_commit.parent_ids();
-            let new_parent_ids = self.new_parents(old_parent_ids);
-            if self.abandoned.contains(&old_commit_id) {
-                // Update the `new_parents` map so descendants are rebased correctly.
-                self.parent_mapping
-                    .insert(old_commit_id.clone(), new_parent_ids.clone());
-                self.update_references(old_commit_id, new_parent_ids, false)?;
-                continue;
-            } else if new_parent_ids == old_parent_ids {
-                // The commit is already in place.
-                continue;
-            }
-
-            // Don't create commit where one parent is an ancestor of another.
-            let head_set: HashSet<_> = self
-                .mut_repo
-                .index()
-                .heads(&mut new_parent_ids.iter())
-                .into_iter()
-                .collect();
-            let new_parents: Vec<_> = new_parent_ids
-                .iter()
-                .filter(|new_parent| head_set.contains(new_parent))
-                .map(|new_parent_id| self.mut_repo.store().get_commit(new_parent_id))
-                .try_collect()?;
-            let new_commit = rebase_commit_with_options(
-                self.settings,
-                self.mut_repo,
-                &old_commit,
-                &new_parents,
-                &self.options,
-            )?;
-            let previous_rebased_value = self
-                .rebased
-                .insert(old_commit_id.clone(), new_commit.id().clone());
-            let previous_mapping_value = self
-                .parent_mapping
-                .insert(old_commit_id.clone(), vec![new_commit.id().clone()]);
-            assert_eq!(
-                (previous_rebased_value, previous_mapping_value),
-                (None, None),
-                "Trying to rebase the same commit {old_commit_id:?} in two different ways",
-            );
-            self.update_references(old_commit_id, vec![new_commit.id().clone()], true)?;
-            return Ok(Some(RebasedDescendant {
-                old_commit,
-                new_commit,
-            }));
+    fn rebase_one(&mut self, old_commit: Commit) -> Result<(), TreeMergeError> {
+        let old_commit_id = old_commit.id().clone();
+        if let Some(new_parent_ids) = self.parent_mapping.get(&old_commit_id).cloned() {
+            // This is a commit that had already been rebased before `self` was created
+            // (i.e. it's part of the input for this rebase). We don't need
+            // to rebase it, but we still want to update branches pointing
+            // to the old commit.
+            self.update_references(old_commit_id, new_parent_ids, false)?;
+            return Ok(());
         }
-        Ok(None)
+        if let Some(divergent_ids) = self.divergent.get(&old_commit_id).cloned() {
+            // Leave divergent commits in place. Don't update `parent_mapping` since we
+            // don't want to rebase descendants either.
+            self.update_references(old_commit_id, divergent_ids, false)?;
+            return Ok(());
+        }
+        let old_parent_ids = old_commit.parent_ids();
+        let new_parent_ids = self.new_parents(old_parent_ids);
+        if self.abandoned.contains(&old_commit_id) {
+            // Update the `new_parents` map so descendants are rebased correctly.
+            self.parent_mapping
+                .insert(old_commit_id.clone(), new_parent_ids.clone());
+            self.update_references(old_commit_id, new_parent_ids, true)?;
+            return Ok(());
+        } else if new_parent_ids == old_parent_ids {
+            // The commit is already in place.
+            return Ok(());
+        }
+
+        // Don't create commit where one parent is an ancestor of another.
+        let head_set: HashSet<_> = self
+            .mut_repo
+            .index()
+            .heads(&mut new_parent_ids.iter())
+            .into_iter()
+            .collect();
+        let new_parents: Vec<_> = new_parent_ids
+            .iter()
+            .filter(|new_parent| head_set.contains(new_parent))
+            .map(|new_parent_id| self.mut_repo.store().get_commit(new_parent_id))
+            .try_collect()?;
+        let new_commit = rebase_commit_with_options(
+            self.settings,
+            self.mut_repo,
+            &old_commit,
+            &new_parents,
+            &self.options,
+        )?;
+        let previous_rebased_value = self
+            .rebased
+            .insert(old_commit_id.clone(), new_commit.id().clone());
+        let previous_mapping_value = self
+            .parent_mapping
+            .insert(old_commit_id.clone(), vec![new_commit.id().clone()]);
+        assert_eq!(
+            (previous_rebased_value, previous_mapping_value),
+            (None, None),
+            "Trying to rebase the same commit {old_commit_id:?} in two different ways",
+        );
+        self.update_references(old_commit_id, vec![new_commit.id().clone()], false)?;
+        Ok(())
     }
 
     pub fn rebase_all(&mut self) -> Result<(), TreeMergeError> {
-        while self.rebase_next()?.is_some() {}
-        // TODO: As the TODO above says, we should probably change the API.
+        while let Some(old_commit) = self.to_visit.pop() {
+            self.rebase_one(old_commit)?;
+        }
         let mut view = self.mut_repo.view().store_view().clone();
         for commit_id in &self.heads_to_remove {
             view.head_ids.remove(commit_id);
@@ -606,9 +604,3 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         Ok(())
     }
 }
-
-#[derive(Debug, PartialEq, Eq, Clone)]
-pub struct RebasedDescendant {
-    pub old_commit: Commit,
-    pub new_commit: Commit,
-}
