@@ -21,7 +21,7 @@ use chrono::DateTime;
 use rand::prelude::*;
 use rand_chacha::ChaCha20Rng;
 
-use crate::backend::{ChangeId, Commit, ObjectId, Signature, Timestamp};
+use crate::backend::{ChangeId, Commit, Signature, Timestamp};
 use crate::fmt_util::binary_prefix;
 use crate::fsmonitor::FsmonitorKind;
 use crate::signing::SignBehavior;
