@@ -14,28 +14,36 @@
 
 #![allow(missing_docs)]
 
-use std::collections::BTreeMap;
+use std::any::Any;
+use std::collections::{BTreeMap, HashMap, HashSet};
 use std::fmt::Debug;
-use std::fs;
 use std::io::{ErrorKind, Write};
 use std::path::{Path, PathBuf};
+use std::time::SystemTime;
+use std::{fs, io};
 
+use itertools::Itertools as _;
 use prost::Message;
 use tempfile::NamedTempFile;
 use thiserror::Error;
 
-use crate::backend::{CommitId, MillisSinceEpoch, ObjectId, Timestamp};
+use crate::backend::{CommitId, MillisSinceEpoch, Timestamp};
 use crate::content_hash::blake2b_hash;
-use crate::file_util::persist_content_addressed_temp_file;
+use crate::file_util::{persist_content_addressed_temp_file, IoResultExt as _, PathError};
 use crate::merge::Merge;
+use crate::object_id::{HexPrefix, ObjectId, PrefixResolution};
 use crate::op_store::{
     OpStore, OpStoreError, OpStoreResult, Operation, OperationId, OperationMetadata, RefTarget,
     RemoteRef, RemoteRefState, RemoteView, View, ViewId, WorkspaceId,
 };
-use crate::{git, op_store};
+use crate::{dag_walk, git, op_store};
+
+// BLAKE2b-512 hash length in bytes
+const OPERATION_ID_LENGTH: usize = 64;
+const VIEW_ID_LENGTH: usize = 64;
 
 #[derive(Debug, Error)]
-#[error("Failed to read {kind} with ID {id}: {err}")]
+#[error("Failed to read {kind} with ID {id}")]
 struct DecodeError {
     kind: &'static str,
     id: String,
@@ -52,6 +60,8 @@ impl From<DecodeError> for OpStoreError {
 #[derive(Debug)]
 pub struct SimpleOpStore {
     path: PathBuf,
+    empty_view_id: ViewId,
+    root_operation_id: OperationId,
 }
 
 impl SimpleOpStore {
@@ -63,15 +73,15 @@ impl SimpleOpStore {
     pub fn init(store_path: &Path) -> Self {
         fs::create_dir(store_path.join("views")).unwrap();
         fs::create_dir(store_path.join("operations")).unwrap();
-        SimpleOpStore {
-            path: store_path.to_owned(),
-        }
+        Self::load(store_path)
     }
 
     /// Load an existing OpStore
     pub fn load(store_path: &Path) -> Self {
         SimpleOpStore {
             path: store_path.to_path_buf(),
+            empty_view_id: ViewId::from_bytes(&[0; VIEW_ID_LENGTH]),
+            root_operation_id: OperationId::from_bytes(&[0; OPERATION_ID_LENGTH]),
         }
     }
 
@@ -85,11 +95,23 @@ impl SimpleOpStore {
 }
 
 impl OpStore for SimpleOpStore {
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
+
     fn name(&self) -> &str {
         Self::name()
     }
 
+    fn root_operation_id(&self) -> &OperationId {
+        &self.root_operation_id
+    }
+
     fn read_view(&self, id: &ViewId) -> OpStoreResult<View> {
+        if *id == self.empty_view_id {
+            return Ok(View::default());
+        }
+
         let path = self.view_path(id);
         let buf = fs::read(path).map_err(|err| io_to_read_error(err, id))?;
 
@@ -119,6 +141,10 @@ impl OpStore for SimpleOpStore {
     }
 
     fn read_operation(&self, id: &OperationId) -> OpStoreResult<Operation> {
+        if *id == self.root_operation_id {
+            return Ok(Operation::make_root(self.empty_view_id.clone()));
+        }
+
         let path = self.operation_path(id);
         let buf = fs::read(path).map_err(|err| io_to_read_error(err, id))?;
 
@@ -128,10 +154,17 @@ impl OpStore for SimpleOpStore {
                 id: id.hex(),
                 err,
             })?;
-        Ok(operation_from_proto(proto))
+        let mut operation = operation_from_proto(proto);
+        if operation.parents.is_empty() {
+            // Repos created before we had the root operation will have an operation without
+            // parents.
+            operation.parents.push(self.root_operation_id.clone());
+        }
+        Ok(operation)
     }
 
     fn write_operation(&self, operation: &Operation) -> OpStoreResult<OperationId> {
+        assert!(!operation.parents.is_empty());
         let temp_file =
             NamedTempFile::new_in(&self.path).map_err(|err| io_to_write_error(err, "operation"))?;
 
@@ -147,6 +180,134 @@ impl OpStore for SimpleOpStore {
             .map_err(|err| io_to_write_error(err, "operation"))?;
         Ok(id)
     }
+
+    fn resolve_operation_id_prefix(
+        &self,
+        prefix: &HexPrefix,
+    ) -> OpStoreResult<PrefixResolution<OperationId>> {
+        let op_dir = self.path.join("operations");
+        let find = || -> io::Result<_> {
+            let matches_root = prefix.matches(&self.root_operation_id);
+            let hex_prefix = prefix.hex();
+            if hex_prefix.len() == OPERATION_ID_LENGTH * 2 {
+                // Fast path for full-length ID
+                if matches_root || op_dir.join(hex_prefix).try_exists()? {
+                    let id = OperationId::from_bytes(prefix.as_full_bytes().unwrap());
+                    return Ok(PrefixResolution::SingleMatch(id));
+                } else {
+                    return Ok(PrefixResolution::NoMatch);
+                }
+            }
+
+            let mut matched = matches_root.then(|| self.root_operation_id.clone());
+            for entry in op_dir.read_dir()? {
+                let Ok(name) = entry?.file_name().into_string() else {
+                    continue; // Skip invalid UTF-8
+                };
+                if !name.starts_with(&hex_prefix) {
+                    continue;
+                }
+                let Ok(id) = OperationId::try_from_hex(&name) else {
+                    continue; // Skip invalid hex
+                };
+                if matched.is_some() {
+                    return Ok(PrefixResolution::AmbiguousMatch);
+                }
+                matched = Some(id);
+            }
+            if let Some(id) = matched {
+                Ok(PrefixResolution::SingleMatch(id))
+            } else {
+                Ok(PrefixResolution::NoMatch)
+            }
+        };
+        find()
+            .context(&op_dir)
+            .map_err(|err| OpStoreError::Other(err.into()))
+    }
+
+    #[tracing::instrument(skip(self))]
+    fn gc(&self, head_ids: &[OperationId], keep_newer: SystemTime) -> OpStoreResult<()> {
+        let to_op_id = |entry: &fs::DirEntry| -> Option<OperationId> {
+            let name = entry.file_name().into_string().ok()?;
+            OperationId::try_from_hex(&name).ok()
+        };
+        let to_view_id = |entry: &fs::DirEntry| -> Option<ViewId> {
+            let name = entry.file_name().into_string().ok()?;
+            ViewId::try_from_hex(&name).ok()
+        };
+        let remove_file_if_not_new = |entry: &fs::DirEntry| -> Result<(), PathError> {
+            let path = entry.path();
+            // Check timestamp, but there's still TOCTOU problem if an existing
+            // file is renewed.
+            let metadata = entry.metadata().context(&path)?;
+            let mtime = metadata.modified().expect("unsupported platform?");
+            if mtime > keep_newer {
+                tracing::trace!(?path, "not removing");
+                Ok(())
+            } else {
+                tracing::trace!(?path, "removing");
+                fs::remove_file(&path).context(&path)
+            }
+        };
+
+        // Reachable objects are resolved without considering the keep_newer
+        // parameter. We could collect ancestors of the "new" operations here,
+        // but more files can be added anyway after that.
+        let read_op = |id: &OperationId| self.read_operation(id).map(|data| (id.clone(), data));
+        let reachable_ops: HashMap<OperationId, Operation> = dag_walk::dfs_ok(
+            head_ids.iter().map(read_op),
+            |(id, _)| id.clone(),
+            |(_, data)| data.parents.iter().map(read_op).collect_vec(),
+        )
+        .try_collect()?;
+        let reachable_views: HashSet<&ViewId> =
+            reachable_ops.values().map(|data| &data.view_id).collect();
+        tracing::info!(
+            reachable_op_count = reachable_ops.len(),
+            reachable_view_count = reachable_views.len(),
+            "collected reachable objects"
+        );
+
+        let prune_ops = || -> Result<(), PathError> {
+            let op_dir = self.path.join("operations");
+            for entry in op_dir.read_dir().context(&op_dir)? {
+                let entry = entry.context(&op_dir)?;
+                let Some(id) = to_op_id(&entry) else {
+                    tracing::trace!(?entry, "skipping invalid file name");
+                    continue;
+                };
+                if reachable_ops.contains_key(&id) {
+                    continue;
+                }
+                // If the operation was added after collecting reachable_views,
+                // its view mtime would also be renewed. So there's no need to
+                // update the reachable_views set to preserve the view.
+                remove_file_if_not_new(&entry)?;
+            }
+            Ok(())
+        };
+        prune_ops().map_err(|err| OpStoreError::Other(err.into()))?;
+
+        let prune_views = || -> Result<(), PathError> {
+            let view_dir = self.path.join("views");
+            for entry in view_dir.read_dir().context(&view_dir)? {
+                let entry = entry.context(&view_dir)?;
+                let Some(id) = to_view_id(&entry) else {
+                    tracing::trace!(?entry, "skipping invalid file name");
+                    continue;
+                };
+                if reachable_views.contains(&id) {
+                    continue;
+                }
+                remove_file_if_not_new(&entry)?;
+            }
+            Ok(())
+        };
+        prune_views().map_err(|err| OpStoreError::Other(err.into()))?;
+
+        Ok(())
+    }
 }
 
 fn io_to_read_error(err: std::io::Error, id: &impl ObjectId) -> OpStoreError {
@@ -251,9 +412,6 @@ fn view_to_proto(view: &View) -> crate::protos::op_store::View {
     for head_id in &view.head_ids {
         proto.head_ids.push(head_id.to_bytes());
     }
-    for head_id in &view.public_head_ids {
-        proto.public_head_ids.push(head_id.to_bytes());
-    }
 
     proto.branches = branch_views_to_proto_legacy(&view.local_branches, &view.remote_views);
 
@@ -293,9 +451,6 @@ fn view_from_proto(proto: crate::protos::op_store::View) -> View {
     for head_id_bytes in proto.head_ids {
         view.head_ids.insert(CommitId::new(head_id_bytes));
     }
-    for head_id_bytes in proto.public_head_ids {
-        view.public_head_ids.insert(CommitId::new(head_id_bytes));
-    }
 
     let (local_branches, remote_views) = branch_views_from_proto_legacy(proto.branches);
     view.local_branches = local_branches;
@@ -528,8 +683,9 @@ mod tests {
     use maplit::{btreemap, hashmap, hashset};
 
     use super::*;
-    use crate::backend::{CommitId, MillisSinceEpoch, ObjectId, Timestamp};
+    use crate::backend::{CommitId, MillisSinceEpoch, Timestamp};
     use crate::content_hash::blake2b_hash;
+    use crate::object_id::ObjectId;
     use crate::op_store::{OperationMetadata, RefTarget, WorkspaceId};
 
     fn create_view() -> View {
@@ -543,8 +699,6 @@ mod tests {
         };
         let head_id1 = CommitId::from_hex("aaa111");
         let head_id2 = CommitId::from_hex("aaa222");
-        let public_head_id1 = CommitId::from_hex("bbb444");
-        let public_head_id2 = CommitId::from_hex("bbb555");
         let branch_main_local_target = RefTarget::normal(CommitId::from_hex("ccc111"));
         let branch_main_origin_target = RefTarget::normal(CommitId::from_hex("ccc222"));
         let branch_deleted_origin_target = RefTarget::normal(CommitId::from_hex("ccc333"));
@@ -558,7 +712,6 @@ mod tests {
         let test_wc_commit_id = CommitId::from_hex("abc222");
         View {
             head_ids: hashset! {head_id1, head_id2},
-            public_head_ids: hashset! {public_head_id1, public_head_id2},
             local_branches: btreemap! {
                 "main".to_string() => branch_main_local_target,
             },
@@ -617,7 +770,7 @@ mod tests {
         // Test exact output so we detect regressions in compatibility
         assert_snapshot!(
             ViewId::new(blake2b_hash(&create_view()).to_vec()).hex(),
-            @"d6d19f3edcb3b2fed6104801b7938e6be1147ab036e9fa81b7624fd5ff0149a6c221c3abb6fb7380c3f37e077a03b234313b44cbb6faca0b8c76f68f24ea7174"
+            @"19af495b5a828c38e933c518a53eb9f7fb7129348a0950d7971b86adf7eb25a13bf45c7f358da1776b27fe6f4bc9134787389d976b768e46ac6cef56bc20c083"
         );
     }
 
@@ -708,12 +861,12 @@ mod tests {
 
     #[test]
     fn test_migrate_git_refs_remote_named_git() {
-        let normal_ref_target = |id_hex: &str| RefTarget::normal(CommitId::from_hex(id_hex));
-        let normal_new_remote_ref = |id_hex: &str| RemoteRef {
+        let normal_ref_target = |id_hex| RefTarget::normal(CommitId::from_hex(id_hex));
+        let normal_new_remote_ref = |id_hex| RemoteRef {
             target: normal_ref_target(id_hex),
             state: RemoteRefState::New,
         };
-        let normal_tracking_remote_ref = |id_hex: &str| RemoteRef {
+        let normal_tracking_remote_ref = |id_hex| RemoteRef {
             target: normal_ref_target(id_hex),
             state: RemoteRefState::Tracking,
         };
