@@ -26,10 +26,10 @@ use crate::backend::{BackendError, MergedTreeId};
 use crate::commit::Commit;
 use crate::fsmonitor::FsmonitorKind;
 use crate::gitignore::GitIgnoreFile;
-use crate::merged_tree::MergedTree;
 use crate::op_store::{OperationId, WorkspaceId};
 use crate::repo_path::{RepoPath, RepoPathBuf};
 use crate::settings::HumanByteSize;
+use crate::store::Store;
 
 /// The trait all working-copy implementations must implement.
 pub trait WorkingCopy {
@@ -63,6 +63,27 @@ pub trait WorkingCopy {
     fn start_mutation(&self) -> Result<Box<dyn LockedWorkingCopy>, WorkingCopyStateError>;
 }
 
+/// The factory which creates and loads a specific type of working copy.
+pub trait WorkingCopyFactory {
+    /// Create a new working copy from scratch.
+    fn init_working_copy(
+        &self,
+        store: Arc<Store>,
+        working_copy_path: PathBuf,
+        state_path: PathBuf,
+        operation_id: OperationId,
+        workspace_id: WorkspaceId,
+    ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError>;
+
+    /// Load an existing working copy.
+    fn load_working_copy(
+        &self,
+        store: Arc<Store>,
+        working_copy_path: PathBuf,
+        state_path: PathBuf,
+    ) -> Box<dyn WorkingCopy>;
+}
+
 /// A working copy that's being modified.
 pub trait LockedWorkingCopy {
     /// Should return `self`. For down-casting purposes.
@@ -83,8 +104,8 @@ pub trait LockedWorkingCopy {
     /// Check out the specified commit in the working copy.
     fn check_out(&mut self, commit: &Commit) -> Result<CheckoutStats, CheckoutError>;
 
-    /// Update to another tree without touching the files in the working copy.
-    fn reset(&mut self, new_tree: &MergedTree) -> Result<(), ResetError>;
+    /// Update to another commit without touching the files in the working copy.
+    fn reset(&mut self, commit: &Commit) -> Result<(), ResetError>;
 
     /// See `WorkingCopy::sparse_patterns()`
     fn sparse_patterns(&self) -> Result<&[RepoPathBuf], WorkingCopyStateError>;
@@ -127,7 +148,7 @@ pub enum SnapshotError {
         target: PathBuf,
     },
     /// Reading or writing from the commit backend failed.
-    #[error("Internal backend error: {0}")]
+    #[error("Internal backend error")]
     InternalBackendError(#[from] BackendError),
     /// A file was larger than the specified maximum file size for new
     /// (previously untracked) files.
@@ -141,7 +162,7 @@ pub enum SnapshotError {
         max_size: HumanByteSize,
     },
     /// Some other error happened while snapshotting the working copy.
-    #[error("{message}: {err}")]
+    #[error("{message}")]
     Other {
         /// Error message.
         message: String,
@@ -211,7 +232,7 @@ pub struct CheckoutStats {
 pub enum CheckoutError {
     /// The current working-copy commit was deleted, maybe by an overly
     /// aggressive GC that happened while the current process was running.
-    #[error("Current working-copy commit not found: {source}")]
+    #[error("Current working-copy commit not found")]
     SourceNotFound {
         /// The underlying error.
         source: Box<dyn std::error::Error + Send + Sync>,
@@ -221,10 +242,10 @@ pub enum CheckoutError {
     #[error("Concurrent checkout")]
     ConcurrentCheckout,
     /// Reading or writing from the commit backend failed.
-    #[error("Internal backend error: {0}")]
+    #[error("Internal backend error")]
     InternalBackendError(#[from] BackendError),
     /// Some other error happened while checking out the working copy.
-    #[error("{message}: {err}")]
+    #[error("{message}")]
     Other {
         /// Error message.
         message: String,
@@ -239,31 +260,32 @@ pub enum CheckoutError {
 pub enum ResetError {
     /// The current working-copy commit was deleted, maybe by an overly
     /// aggressive GC that happened while the current process was running.
-    #[error("Current working-copy commit not found: {source}")]
+    #[error("Current working-copy commit not found")]
     SourceNotFound {
         /// The underlying error.
         source: Box<dyn std::error::Error + Send + Sync>,
     },
     /// Reading or writing from the commit backend failed.
-    #[error("Internal error: {0}")]
+    #[error("Internal error")]
     InternalBackendError(#[from] BackendError),
     /// Some other error happened while checking out the working copy.
-    #[error("{message}: {err}")]
+    #[error("{message}")]
     Other {
         /// Error message.
         message: String,
-        #[source]
         /// The underlying error.
+        #[source]
         err: Box<dyn std::error::Error + Send + Sync>,
     },
 }
 
 /// An error while reading the working copy state.
 #[derive(Debug, Error)]
-#[error("{message}: {err}")]
+#[error("{message}")]
 pub struct WorkingCopyStateError {
     /// Error message.
     pub message: String,
     /// The underlying error.
+    #[source]
     pub err: Box<dyn std::error::Error + Send + Sync>,
 }
