@@ -16,7 +16,7 @@ use std::fs;
 use std::sync::Arc;
 
 use assert_matches::assert_matches;
-use jj_lib::backend::{CommitId, ObjectId as _};
+use jj_lib::backend::{ChangeId, CommitId};
 use jj_lib::commit::Commit;
 use jj_lib::commit_builder::CommitBuilder;
 use jj_lib::default_index::{
@@ -24,6 +24,8 @@ use jj_lib::default_index::{
     DefaultMutableIndex, DefaultReadonlyIndex, IndexPosition,
 };
 use jj_lib::index::Index as _;
+use jj_lib::object_id::{HexPrefix, ObjectId as _, PrefixResolution};
+use jj_lib::op_store::{RefTarget, RemoteRef};
 use jj_lib::repo::{MutableRepo, ReadonlyRepo, Repo};
 use jj_lib::settings::UserSettings;
 use testutils::test_backend::TestBackend;
@@ -312,6 +314,54 @@ fn test_index_commits_previous_operations() {
     assert_eq!(generation_number(index, commit_c.id()), 3);
 }
 
+#[test]
+fn test_index_commits_hidden_but_referenced() {
+    // Test that hidden-but-referenced commits are indexed.
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    // Remote branches are usually visible at a certain point in operation
+    // history, but that's not guaranteed if old operations have been discarded.
+    // This can also happen if imported remote branches get immediately
+    // abandoned because the other branch has moved.
+    let mut tx = repo.start_transaction(&settings);
+    let commit_a = write_random_commit(tx.mut_repo(), &settings);
+    let commit_b = write_random_commit(tx.mut_repo(), &settings);
+    let commit_c = write_random_commit(tx.mut_repo(), &settings);
+    tx.mut_repo().remove_head(commit_a.id());
+    tx.mut_repo().remove_head(commit_b.id());
+    tx.mut_repo().remove_head(commit_c.id());
+    tx.mut_repo().set_remote_branch(
+        "branch",
+        "origin",
+        RemoteRef {
+            target: RefTarget::from_legacy_form(
+                [commit_a.id().clone()],
+                [commit_b.id().clone(), commit_c.id().clone()],
+            ),
+            state: jj_lib::op_store::RemoteRefState::New,
+        },
+    );
+    let repo = tx.commit("test");
+
+    // All commits should be indexed
+    assert!(repo.index().has_id(commit_a.id()));
+    assert!(repo.index().has_id(commit_b.id()));
+    assert!(repo.index().has_id(commit_c.id()));
+
+    // Delete index from disk
+    let default_index_store: &DefaultIndexStore =
+        repo.index_store().as_any().downcast_ref().unwrap();
+    default_index_store.reinit().unwrap();
+
+    let repo = load_repo_at_head(&settings, repo.repo_path());
+    // All commits should be reindexed
+    assert!(repo.index().has_id(commit_a.id()));
+    assert!(repo.index().has_id(commit_b.id()));
+    assert!(repo.index().has_id(commit_c.id()));
+}
+
 #[test]
 fn test_index_commits_incremental() {
     let settings = testutils::user_settings();
@@ -641,3 +691,106 @@ fn test_index_store_type() {
     );
     assert_eq!(as_readonly_composite(&repo).num_commits(), 1);
 }
+
+#[test]
+fn test_change_id_index() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+
+    let root_commit = repo.store().root_commit();
+    let mut commit_number = 0;
+    let mut commit_with_change_id = |change_id| {
+        commit_number += 1;
+        tx.mut_repo()
+            .new_commit(
+                &settings,
+                vec![root_commit.id().clone()],
+                root_commit.tree_id().clone(),
+            )
+            .set_change_id(ChangeId::from_hex(change_id))
+            .set_description(format!("commit {commit_number}"))
+            .write()
+            .unwrap()
+    };
+    let commit_1 = commit_with_change_id("abbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb");
+    let commit_2 = commit_with_change_id("aaaaabbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb");
+    let commit_3 = commit_with_change_id("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
+    let commit_4 = commit_with_change_id("bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb");
+    let commit_5 = commit_with_change_id("bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb");
+
+    let index_for_heads = |commits: &[&Commit]| {
+        tx.repo()
+            .mutable_index()
+            .change_id_index(&mut commits.iter().map(|commit| commit.id()))
+    };
+    let change_id_index = index_for_heads(&[&commit_1, &commit_2, &commit_3, &commit_4, &commit_5]);
+    let prefix_len =
+        |commit: &Commit| change_id_index.shortest_unique_prefix_len(commit.change_id());
+    assert_eq!(prefix_len(&root_commit), 1);
+    assert_eq!(prefix_len(&commit_1), 2);
+    assert_eq!(prefix_len(&commit_2), 6);
+    assert_eq!(prefix_len(&commit_3), 6);
+    assert_eq!(prefix_len(&commit_4), 1);
+    assert_eq!(prefix_len(&commit_5), 1);
+    let resolve_prefix =
+        |prefix: &str| change_id_index.resolve_prefix(&HexPrefix::new(prefix).unwrap());
+    // Ambiguous matches
+    assert_eq!(resolve_prefix("a"), PrefixResolution::AmbiguousMatch);
+    assert_eq!(resolve_prefix("aaaaa"), PrefixResolution::AmbiguousMatch);
+    // Exactly the necessary length
+    assert_eq!(
+        resolve_prefix("0"),
+        PrefixResolution::SingleMatch(vec![root_commit.id().clone()])
+    );
+    assert_eq!(
+        resolve_prefix("aaaaaa"),
+        PrefixResolution::SingleMatch(vec![commit_3.id().clone()])
+    );
+    assert_eq!(
+        resolve_prefix("aaaaab"),
+        PrefixResolution::SingleMatch(vec![commit_2.id().clone()])
+    );
+    assert_eq!(
+        resolve_prefix("ab"),
+        PrefixResolution::SingleMatch(vec![commit_1.id().clone()])
+    );
+    assert_eq!(
+        resolve_prefix("b"),
+        PrefixResolution::SingleMatch(vec![commit_5.id().clone(), commit_4.id().clone()])
+    );
+    // Longer than necessary
+    assert_eq!(
+        resolve_prefix("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
+        PrefixResolution::SingleMatch(vec![commit_3.id().clone()])
+    );
+    // No match
+    assert_eq!(resolve_prefix("ba"), PrefixResolution::NoMatch);
+
+    // Test with a revset containing only some of the commits. We should get shorter
+    // prefixes and be able to resolve shorter prefixes.
+    let change_id_index = index_for_heads(&[&commit_1, &commit_2]);
+    let prefix_len =
+        |commit: &Commit| change_id_index.shortest_unique_prefix_len(commit.change_id());
+    assert_eq!(prefix_len(&commit_1), 2);
+    assert_eq!(prefix_len(&commit_2), 2);
+    assert_eq!(prefix_len(&commit_3), 6);
+    let resolve_prefix =
+        |prefix: &str| change_id_index.resolve_prefix(&HexPrefix::new(prefix).unwrap());
+    assert_eq!(
+        resolve_prefix("0"),
+        PrefixResolution::SingleMatch(vec![root_commit.id().clone()])
+    );
+    assert_eq!(
+        resolve_prefix("aa"),
+        PrefixResolution::SingleMatch(vec![commit_2.id().clone()])
+    );
+    assert_eq!(
+        resolve_prefix("ab"),
+        PrefixResolution::SingleMatch(vec![commit_1.id().clone()])
+    );
+    assert_eq!(resolve_prefix("a"), PrefixResolution::AmbiguousMatch);
+    assert_eq!(resolve_prefix("b"), PrefixResolution::NoMatch);
+}
